﻿//******************************************************************************************************
//  CalibrationResults.cs
//
//  Copyright © 2013, Kevin D. Jones.  All Rights Reserved.
//
//  This file is licensed to you under the Eclipse Public License -v 1.0 (the "License"); you may
//  not use this file except in compliance with the License. You may obtain a copy of the License at:
//
//      http://www.opensource.org/licenses/eclipse-1.0.php
//
//  Unless agreed to in writing, the subject software distributed under the License is distributed on an
//  "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the
//  License for the specific language governing permissions and limitations.
//
//  Code Modification History:
//  ----------------------------------------------------------------------------------------------------
//  07/26/2013 - Kevin D. Jones
//       Generated original version of source code.
//
//******************************************************************************************************
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Serialization;
using SynchrophasorAnalytics.Measurements;

namespace SynchrophasorAnalytics.Calibration
{
    [Serializable()]
    [XmlRoot("CalibrationResults")]
    public class CalibrationResults
    {
        private int m_measurementID;
        private PhasorType m_phasorType;
        private double m_aPhaseRcf;
        private double m_bPhaseRcf;
        private double m_cPhaseRcf;
        private double m_aPhasePacf; 
        private double m_bPhasePacf;
        private double m_cPhasePacf;

        [XmlAttribute("ID")]
        public int MeasurementID
        {
            get
            {
                return m_measurementID;
            }
            set
            {
                m_measurementID = value;
            }
        }

        [XmlAttribute("Type")]
        public PhasorType MeasurementType 
        {
            get
            {
                return m_phasorType;
            }
            set
            {
                m_phasorType = value;
            } 
        }

        [XmlAttribute("APhaseRCF")]
        public double APhaseRCF
        {
            get
            {
                return m_aPhaseRcf;
            }
            set
            {
                m_aPhaseRcf = value;
            }
        }

        [XmlAttribute("BPhaseRCF")]
        public double BPhaseRCF
        {
            get
            {
                return m_bPhaseRcf;
            }
            set
            {
                m_bPhaseRcf = value;
            }
        }

        [XmlAttribute("CPhaseRCF")]
        public double CPhaseRCF
        {
            get
            {
                return m_cPhaseRcf;
            }
            set
            {
                m_cPhaseRcf = value;
            }
        }

        [XmlAttribute("APhasePACF")]
        public double APhasePACF
        {
            get
            {
                return m_aPhasePacf;
            }
            set
            {
                m_aPhasePacf = value;
            }
        }

        [XmlAttribute("BPhasePACF")]
        public double BPhasePACF
        {
            get
            {
                return m_bPhasePacf;
            }
            set
            {
                m_bPhasePacf = value;
            }
        }

        [XmlAttribute("CPhasePACF")]
        public double CPhasePACF
        {
            get
            {
                return m_cPhasePacf;
            }
            set
            {
                m_cPhasePacf = value;
            }
        }

        public CalibrationResults()
            :this(1, PhasorType.CurrentPhasor, 1, 1, 1, 0, 0, 0)
        {
        }

        public CalibrationResults(int measurementID, PhasorType phasorType, double aPhaseRcf, double bPhaseRcf, double cPhaseRcf, double aPhasePacf, double bPhasePacf, double cPhasePacf)
        {
            m_measurementID = measurementID;
            m_phasorType = phasorType;
            m_aPhaseRcf = aPhaseRcf;
            m_bPhaseRcf = bPhaseRcf;
            m_cPhaseRcf = cPhaseRcf;
            m_aPhasePacf = aPhasePacf;
            m_bPhasePacf = bPhasePacf;
            m_cPhasePacf = cPhasePacf;
        }
    }
}
