﻿//******************************************************************************************************
//  SignalToNoiseRatioIOMapping.cs
//
//  Copyright © 2013, Kevin D. Jones.  All Rights Reserved.
//
//  This file is licensed to you under the Eclipse Public License -v 1.0 (the "License"); you may
//  not use this file except in compliance with the License. You may obtain a copy of the License at:
//
//      http://www.opensource.org/licenses/eclipse-1.0.php
//
//  Unless agreed to in writing, the subject software distributed under the License is distributed on an
//  "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the
//  License for the specific language governing permissions and limitations.
//
//  Code Modification History:
//  ----------------------------------------------------------------------------------------------------
//  08/26/2013 - Kevin D. Jones
//       Generated original version of source code.
//
//******************************************************************************************************
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Serialization;

namespace SynchrophasorAnalytics.DataConditioning
{
    
    [Serializable()]
    public class SignalToNoiseRatioIOMapping
    {
        #region [ Private Members ]

        private string m_inputKey;
        private string m_outputKey;
        private SignalToNoiseRatioMeasurementType m_signalType;
        private List<double> m_movingWindow;
        private List<double> m_unwrappedMovingWindow;
        private double m_signalToNoiseRatio;
        private int m_currentKValue;

        #endregion

        #region [ Properties ]

        [XmlAttribute("Input")]
        public string InputKey
        {
            get
            {
                return m_inputKey;
            }
            set
            {
                m_inputKey = value;
            }
        }

        [XmlAttribute("Output")]
        public string OutputKey
        {
            get
            {
                return m_outputKey;
            }
            set
            {
                m_outputKey = value;
            }
        }

        [XmlAttribute("Type")]
        public SignalToNoiseRatioMeasurementType SignalType
        {
            get
            {
                return m_signalType;
            }
            set
            {
                m_signalType = value;
            }
        }

        [XmlIgnore()]
        public List<double> Data
        {
            get
            {
                return m_movingWindow;
            }
            set
            {
                m_movingWindow = value;
            }
        }

        [XmlIgnore()]
        public List<double> UnwrappedData
        {
            get
            {
                return m_unwrappedMovingWindow;
            }
            set
            {
                m_unwrappedMovingWindow = value;
            }
        }

        [XmlAttribute("SNR")]
        public double SNR
        {
            get
            {
                return m_signalToNoiseRatio;
            }
            set
            {
                m_signalToNoiseRatio = value;
            }
        }

        [XmlIgnore()]
        public int CurrentKValue
        {
            get
            {
                return m_currentKValue;   
            }
            set
            {
                m_currentKValue = value;
            }
        }

        #endregion

        #region [ Constructors ]

        /// <summary>
        /// Default constructor for the <see cref="SignalToNoiseRatioIOMapping"/> class. 
        /// </summary>
        public SignalToNoiseRatioIOMapping()
            :this("Undefined", "Undefined", SignalToNoiseRatioMeasurementType.Magnitude)
        {
        }

        /// <summary>
        /// Designated constructor for the <see cref="SignalToNoiseRatioIOMapping"/> class.
        /// </summary>
        /// <param name="inputKey">The input measurement key for the signal.</param>
        /// <param name="outputKey">The corresponding output measurement key for the resultant SNR.</param>
        /// <param name="signalType">The type of the signal, either Magnitude or Angle</param>
        public SignalToNoiseRatioIOMapping(string inputKey, string outputKey, SignalToNoiseRatioMeasurementType signalType)
        {
            m_inputKey = inputKey;
            m_outputKey = outputKey;
            m_signalType = signalType;
            m_movingWindow = new List<double>();
            m_unwrappedMovingWindow = new List<double>();
            m_currentKValue = 0;
        }

        #endregion
    }

}
