﻿//******************************************************************************************************
//  VertexAdjacencyList.cs
//
//  Copyright © 2013, Kevin D. Jones.  All Rights Reserved.
//
//  This file is licensed to you under the Eclipse Public License -v 1.0 (the "License"); you may
//  not use this file except in compliance with the License. You may obtain a copy of the License at:
//
//      http://www.opensource.org/licenses/eclipse-1.0.php
//
//  Unless agreed to in writing, the subject software distributed under the License is distributed on an
//  "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the
//  License for the specific language governing permissions and limitations.
//
//  Code Modification History:
//  ----------------------------------------------------------------------------------------------------
//  06/01/2013 - Kevin D. Jones
//       Generated original version of source code.
//
//******************************************************************************************************
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using SynchrophasorAnalytics.Modeling;

namespace SynchrophasorAnalytics.Graph
{
    public class VertexAdjacencyList
    {
        private List<VertexAdjacencyRow> m_adjacencyRows;

        public List<VertexAdjacencyRow> Rows
        {
            get
            {
                return m_adjacencyRows;
            }
            set
            {
                m_adjacencyRows = value;
            }
        }

        public VertexAdjacencyList()
            :this(new List<VertexAdjacencyRow>())
        {
        }

        public VertexAdjacencyList(List<VertexAdjacencyRow> adjacencyRows)
        {
            m_adjacencyRows = adjacencyRows;
        }

        public override string ToString()
        {
            if (m_adjacencyRows.Count != 0)
            {
                StringBuilder stringBuilder = new StringBuilder();

                List<int> dividerIndices = new List<int>();

                foreach (VertexAdjacencyRow row in m_adjacencyRows)
                {
                    dividerIndices.Add(row.ToString().IndexOf('|'));
                }

                int dividerIndexMaximum = dividerIndices.Max();

                foreach (VertexAdjacencyRow row in m_adjacencyRows)
                {
                    int lengthOfWhiteSpaceToAppend = dividerIndexMaximum - row.ToString().IndexOf('|');

                    string whiteSpaceToAppend = "";

                    for (int i = 0; i < lengthOfWhiteSpaceToAppend; i++)
                    {
                        whiteSpaceToAppend += " ";
                    }

                    stringBuilder.AppendLine(whiteSpaceToAppend + row.ToString());
                }

                return stringBuilder.ToString();
            }
            else
            {
                return "(null set)";
            }
        }

        public void Sort()
        {
            foreach (VertexAdjacencyRow row in m_adjacencyRows)
            {
                row.Sort();
            }
        }

        public void RemoveDuplicateVertices()
        {
            foreach (VertexAdjacencyRow row in m_adjacencyRows)
            {
                row.RemoveDuplicateVertices();
            }
        }

        public void MergeRows(VertexAdjacencyRow source, VertexAdjacencyRow target)
        {
            
        }

        public void RemoveRow(VertexAdjacencyRow row)
        {
            m_adjacencyRows.Remove(row);
        }

        public void AddRow(VertexAdjacencyRow row)
        {

        }

        public VertexAdjacencyRow RowWithHeader(VertexCluster header)
        {
            foreach (VertexAdjacencyRow row in m_adjacencyRows)
            {
                if (row.Header.Equals(header))
                {
                    return row;
                }
            }

            return null;
        }

        
    }
}
