﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Meta.Numerics;

namespace SynchrophasorAnalytics.Matrices
{
    public partial class SimpleMatrix
    {
        #region [ Private Members ]

        private int m_rowCount;
        private int m_columnCount;

        private Complex[,] m_elements;

        #endregion

        #region [ Properties ]

        public int RowCount
        {
            get
            {
                return m_rowCount;
            }
        }

        public int ColumnCount
        {
            get
            {
                return m_columnCount;
            }
        }

        public Complex[,] Elements
        {
            get
            {
                return m_elements;
            }
            set
            {
                m_elements = value;
            }
        }

        #endregion

        #region [ Constructors ]

        public SimpleMatrix(int rowCount, int columnCount)
            :this(rowCount, columnCount, 0)
        {
        }

        public SimpleMatrix(int rowCount, int columnCount, double value)
            :this(rowCount, columnCount, new Complex(value, 0))
        {
        }

        public SimpleMatrix(int rowCount, int columnCount, Complex value)
        {
            if (rowCount > 0)
            {
                m_rowCount = rowCount;
            }
            else
            {
                throw new Exception("Matrix row count must be non-zero positive.");
            }

            if (columnCount > 0)
            {
                m_columnCount = columnCount;
            }
            else
            {
                throw new Exception("Matrix column count must be non-zero positive.");
            }

            m_elements = new Complex[m_rowCount, m_columnCount];

            for (int i = 0; i < m_rowCount; i++)
            {
                for (int j = 0; j < m_columnCount; j++)
                {
                    m_elements[i, j] = value;
                }
            }
        }

        #endregion

        #region [ Public Methods ]

        public SimpleMatrix Transpose()
        {
            return new SimpleMatrix(1, 1);
        }

        public static SimpleMatrix Eye(int dimension)
        {
            SimpleMatrix identity = new SimpleMatrix(dimension, dimension);

            for (int i = 0; i < identity.RowCount; i++)
            {
                identity.Elements[i, i] = new Complex(1, 0);
            }

            return identity;
        }

        public override string ToString()
        {
            StringBuilder stringBuilder = new StringBuilder();

            stringBuilder.AppendLine("--------------------------------------------------------------------------------");

            for (int i = 0; i < m_rowCount; i++)
            {
                string matrixRow = "";

                for (int j = 0; j < m_columnCount; j++)
                {
                    matrixRow += m_elements[i, j].ToString() + " | ";
                }

                stringBuilder.AppendLine(matrixRow);
            }

            return stringBuilder.ToString();
        }

        #endregion
    }
}
