﻿//******************************************************************************************************
//  IImpedance.cs
//
//  Copyright © 2013, Kevin D. Jones.  All Rights Reserved.
//
//  This file is licensed to you under the Eclipse Public License -v 1.0 (the "License"); you may
//  not use this file except in compliance with the License. You may obtain a copy of the License at:
//
//      http://www.opensource.org/licenses/eclipse-1.0.php
//
//  Unless agreed to in writing, the subject software distributed under the License is distributed on an
//  "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the
//  License for the specific language governing permissions and limitations.
//
//  Code Modification History:
//  ----------------------------------------------------------------------------------------------------
//  06/01/2013 - Kevin D. Jones
//       Generated original version of source code.
//
//******************************************************************************************************
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Extreme.Mathematics;
using Extreme.Mathematics.LinearAlgebra;
using Extreme.Mathematics.LinearAlgebra.Complex;

namespace SynchrophasorAnalytics.Modeling
{
    public interface IImpedance
    {
        /// <summary>
        /// The complex (R + jX) series impedance.
        /// </summary>
        DoubleComplex PositiveSequenceSeriesImpedance
        {
            get;
        }

        /// <summary>
        /// The complex 1/(R + jX) series admittance.
        /// </summary>
        DoubleComplex PositiveSequenceSeriesAdmittance
        {
            get;
        }

        /// <summary>
        /// The complex (G + jB) shunt susceptance.
        /// </summary>
        DoubleComplex PositiveSequenceShuntSusceptance
        {
            get;
        }
        
        /// <summary>
        /// The complex (G + jB) shunt susceptance. for a single side of the series branch. (divided by 2)
        /// </summary>
        DoubleComplex PositiveSequenceSingleSidedShuntSusceptance
        {
            get;
        }

        /// <summary>
        /// The three phase complex (R +jX) series impedance as a 3 x 3 complex matrix.
        /// </summary>
        ComplexMatrix ThreePhaseSeriesImpedance
        {
            get;
        }

        /// <summary>
        /// The three phase complex 1/(R +jX) series admittance as a 3 x 3 complex matrix.
        /// </summary>
        ComplexMatrix ThreePhaseSeriesAdmittance
        {
            get;
        }

        /// <summary>
        /// The three phase complex (G + JB) shunt susceptance as a 3 x 3 complex matrix.
        /// </summary>
        ComplexMatrix ThreePhaseShuntSusceptance
        {
            get;
        }

        /// <summary>
        /// The three phase complex (G + JB) shunt susceptance as a 3 x 3 complex matrix for a single side of the series branch. (divided by 2)
        /// </summary>
        ComplexMatrix ThreePhaseSingleSidedShuntSusceptance
        {
            get;
        }

    }
}
