﻿//******************************************************************************************************
//  BreakerStatusDataGridUserControl.cs
//
//  Copyright © 2013, Kevin D. Jones.  All Rights Reserved.
//
//  This file is licensed to you under the Eclipse Public License -v 1.0 (the "License"); you may
//  not use this file except in compliance with the License. You may obtain a copy of the License at:
//
//      http://www.opensource.org/licenses/eclipse-1.0.php
//
//  Unless agreed to in writing, the subject software distributed under the License is distributed on an
//  "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the
//  License for the specific language governing permissions and limitations.
//
//  Code Modification History:
//  ----------------------------------------------------------------------------------------------------
//  07/15/2013 - Kevin D. Jones
//       Generated original version of source code.
//
//******************************************************************************************************
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Configuration;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using NetworkModelBuilder.DataModels;
using FirstFloor.ModernUI.Windows.Controls;
using SynchrophasorAnalytics.Networks;
using SynchrophasorAnalytics.Measurements;

namespace NetworkModelBuilder.DataTables
{
    /// <summary>
    /// Interaction logic for BreakerStatusDataGridUserControl.xaml
    /// </summary>
    public partial class BreakerStatusDataGridUserControl : UserControl
    {
        private ObservableCollection<BreakerStatusDataModel> m_breakerStatusData;
        private BreakerStatusDataModel m_editedBreakerStatus;

        public BreakerStatusDataGridUserControl()
        {
            InitializeComponent(); 
            BreakerStatusDataGrid.CellEditEnding += BreakerStatusDataGrid_CellEditEnding;
            BreakerStatusDataGrid.CurrentCellChanged += BreakerStatusDataGrid_CurrentCellChanged;
            m_breakerStatusData = new ObservableCollection<BreakerStatusDataModel>();
            BreakerStatusDataGrid.DataContext = m_breakerStatusData;
            LoadCurrentBreakerStatusView(LoadNetworkModel());
        }

        private void BreakerStatusDataGrid_CurrentCellChanged(object sender, EventArgs e)
        {
            if (m_editedBreakerStatus != null)
            {
                Network network = LoadNetworkModel();

                foreach (BreakerStatus breakerStatus in network.Model.BreakerStatuses)
                {
                    if (breakerStatus.InternalID == m_editedBreakerStatus.InternalID)
                    {
                        breakerStatus.Number = m_editedBreakerStatus.Number;
                        breakerStatus.Name = m_editedBreakerStatus.Name;
                        breakerStatus.Description = m_editedBreakerStatus.Description;
                        breakerStatus.Key = m_editedBreakerStatus.MeasurementKey;
                        breakerStatus.IsEnabled = m_editedBreakerStatus.IsEnabled;
                        breakerStatus.BitPosition = m_editedBreakerStatus.BitPosition;
                    }
                }

                network.SerializeToXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);

                LoadCurrentBreakerStatusView(LoadNetworkModel());
            }

            m_editedBreakerStatus = null;
        }

        private void BreakerStatusDataGrid_CellEditEnding(object sender, DataGridCellEditEndingEventArgs e)
        {
            if (e.EditAction == DataGridEditAction.Commit)
            {
                m_editedBreakerStatus = e.Row.Item as BreakerStatusDataModel;
            }
        }

        private void AddButton_Click(object sender, RoutedEventArgs e)
        {
            Network network = LoadNetworkModel();

            // Figure out which InternalID to use
            List<int> breakerStatusInternalIds = new List<int>();
            foreach (BreakerStatus breakerStatus in network.Model.BreakerStatuses)
            {
                breakerStatusInternalIds.Add(breakerStatus.InternalID);
            }
            int newBreakerStatusInternalID = breakerStatusInternalIds.Max() + 1;

            network.Model.BreakerStatuses.Add(new BreakerStatus()
            {
                InternalID = newBreakerStatusInternalID,
                Number = 1,
                Name = "Breaker Status Name",
                Description = "Breaker Status Description",
                IsEnabled = true,
                Key = "Undefined",
                BitPosition = BreakerStatusBit.PSV58
            });

            network.SerializeToXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);

            LoadCurrentBreakerStatusView(LoadNetworkModel());
        }

        private void RefreshButton_Click(object sender, RoutedEventArgs e)
        {
            LoadCurrentBreakerStatusView(LoadNetworkModel());
        }

        private void DeleteButton_Click(object sender, RoutedEventArgs e)
        {
            BreakerStatusDataModel selectedItem = (BreakerStatusDataModel)BreakerStatusDataGrid.SelectedItem;

            if (selectedItem != null)
            {
                StringBuilder stringBuilder = new StringBuilder();
                stringBuilder.AppendFormat("Are you sure you want to delete the Breaker Status?{0}", Environment.NewLine);
                stringBuilder.AppendLine();
                stringBuilder.AppendFormat("InternalID: " + selectedItem.InternalID.ToString() + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Number: " + selectedItem.Number.ToString() + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Name: " + selectedItem.Name + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Description: " + selectedItem.Description + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Measurement Key: " + selectedItem.MeasurementKey + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Bit Position: " + selectedItem.BitPosition + "{0}", Environment.NewLine);

                MessageBoxButton confirmDeletionButtons = MessageBoxButton.YesNo;
                var confirmDeletionResult = ModernDialog.ShowMessage(stringBuilder.ToString(), "Confirm Deletion", confirmDeletionButtons);

                if (confirmDeletionResult.ToString().Equals("Yes"))
                {
                    Network network = LoadNetworkModel();

                    network.Model.BreakerStatuses.RemoveAll(x => x.InternalID == selectedItem.InternalID);

                    network.SerializeToXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);

                    LoadCurrentBreakerStatusView(LoadNetworkModel());
                }
            }
        }

        private void BreakerStatusUserControl_Loaded(object sender, RoutedEventArgs e)
        {
        }

        private void LoadCurrentBreakerStatusView(Network network)
        {
            // Clear the current view
            m_breakerStatusData.Clear();

            foreach (BreakerStatus breakerStatus in network.Model.BreakerStatuses)
            {
                m_breakerStatusData.Add(new BreakerStatusDataModel(breakerStatus.InternalID, breakerStatus.Number, breakerStatus.Name, breakerStatus.Description, breakerStatus.IsEnabled, breakerStatus.Key, breakerStatus.BitPosition));
            }
        }

        private Network LoadNetworkModel()
        {
            Network network = Network.DeserializeFromXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);
            network.Initialize();
            return network;
        }
    }
}
