﻿//******************************************************************************************************
//  CurrentPhasorDataGridUserControl.cs
//
//  Copyright © 2013, Kevin D. Jones.  All Rights Reserved.
//
//  This file is licensed to you under the Eclipse Public License -v 1.0 (the "License"); you may
//  not use this file except in compliance with the License. You may obtain a copy of the License at:
//
//      http://www.opensource.org/licenses/eclipse-1.0.php
//
//  Unless agreed to in writing, the subject software distributed under the License is distributed on an
//  "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the
//  License for the specific language governing permissions and limitations.
//
//  Code Modification History:
//  ----------------------------------------------------------------------------------------------------
//  07/15/2013 - Kevin D. Jones
//       Generated original version of source code.
//
//******************************************************************************************************
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Configuration;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using NetworkModelBuilder.DataModels;
using FirstFloor.ModernUI.Windows.Controls;
using SynchrophasorAnalytics.Networks;
using SynchrophasorAnalytics.Modeling;
using SynchrophasorAnalytics.Measurements;

namespace NetworkModelBuilder.DataTables
{
    /// <summary>
    /// Interaction logic for CurrentPhasorDataGridUserControl.xaml
    /// </summary>
    public partial class CurrentPhasorDataGridUserControl : UserControl
    {
        private ObservableCollection<CurrentPhasorDataModel> m_currentPhasorData;
        private CurrentPhasorDataModel m_editedCurrentPhasor;

        public CurrentPhasorDataGridUserControl()
        {
            InitializeComponent();
            CurrentPhasorDataGrid.CellEditEnding += CurrentPhasorDataGrid_CellEditEnding;
            CurrentPhasorDataGrid.CurrentCellChanged += CurrentPhasorDataGrid_CurrentCellChanged;
            m_currentPhasorData = new ObservableCollection<CurrentPhasorDataModel>();
            CurrentPhasorDataGrid.DataContext = m_currentPhasorData;
            LoadCurrentPhasorView(LoadNetworkModel());
        }

        private void CurrentPhasorDataGrid_CurrentCellChanged(object sender, EventArgs e)
        {
            if (m_editedCurrentPhasor != null)
            {
                Network network = LoadNetworkModel();

                foreach (CurrentPhasorGroup currentPhasorGroup in network.Model.Currents)
                {
                    if (currentPhasorGroup.InternalID == m_editedCurrentPhasor.InternalID)
                    {
                        currentPhasorGroup.Number = m_editedCurrentPhasor.Number;
                        currentPhasorGroup.Name = m_editedCurrentPhasor.Name;
                        currentPhasorGroup.Description = m_editedCurrentPhasor.Description;
                        currentPhasorGroup.IsEnabled = m_editedCurrentPhasor.IsEnabled;
                        currentPhasorGroup.UseStatusFlagForRemovingMeasurements = m_editedCurrentPhasor.UseStatusFlag;
                        currentPhasorGroup.PositiveSequence.Measurement.MeasurementVariance = m_editedCurrentPhasor.PositiveSequenceVariance;
                        currentPhasorGroup.PhaseA.Measurement.MeasurementVariance = m_editedCurrentPhasor.APhaseVariance;
                        currentPhasorGroup.PhaseB.Measurement.MeasurementVariance = m_editedCurrentPhasor.BPhaseVariance;
                        currentPhasorGroup.PhaseC.Measurement.MeasurementVariance = m_editedCurrentPhasor.CPhaseVariance;
                        currentPhasorGroup.PositiveSequence.Measurement.InstrumentTransformerCalibrationSetting = m_editedCurrentPhasor.InstrumentTransformerCalibrationSetting;
                        currentPhasorGroup.PhaseA.Measurement.InstrumentTransformerCalibrationSetting = m_editedCurrentPhasor.InstrumentTransformerCalibrationSetting;
                        currentPhasorGroup.PhaseB.Measurement.InstrumentTransformerCalibrationSetting = m_editedCurrentPhasor.InstrumentTransformerCalibrationSetting;
                        currentPhasorGroup.PhaseC.Measurement.InstrumentTransformerCalibrationSetting = m_editedCurrentPhasor.InstrumentTransformerCalibrationSetting;
                        currentPhasorGroup.PositiveSequence.Measurement.RCF = m_editedCurrentPhasor.PositiveSequenceRCF;
                        currentPhasorGroup.PhaseA.Measurement.RCF = m_editedCurrentPhasor.APhaseRCF;
                        currentPhasorGroup.PhaseB.Measurement.RCF = m_editedCurrentPhasor.BPhaseRCF;
                        currentPhasorGroup.PhaseC.Measurement.RCF = m_editedCurrentPhasor.CPhaseRCF;
                        currentPhasorGroup.PositiveSequence.Measurement.PACF = m_editedCurrentPhasor.PositiveSequencePACF;
                        currentPhasorGroup.PhaseA.Measurement.PACF = m_editedCurrentPhasor.APhasePACF;
                        currentPhasorGroup.PhaseB.Measurement.PACF = m_editedCurrentPhasor.BPhasePACF;
                        currentPhasorGroup.PhaseC.Measurement.PACF = m_editedCurrentPhasor.CPhasePACF;
                        currentPhasorGroup.PositiveSequence.Measurement.MagnitudeKey = m_editedCurrentPhasor.PositiveSequenceMagnitudeMeasurementKey;
                        currentPhasorGroup.PositiveSequence.Measurement.AngleKey = m_editedCurrentPhasor.PositiveSequenceAngleMeasurementKey;
                        currentPhasorGroup.PhaseA.Measurement.MagnitudeKey = m_editedCurrentPhasor.APhaseMagnitudeMeasurementKey;
                        currentPhasorGroup.PhaseA.Measurement.AngleKey = m_editedCurrentPhasor.APhaseAngleMeasurementKey;
                        currentPhasorGroup.PhaseB.Measurement.MagnitudeKey = m_editedCurrentPhasor.BPhaseMagnitudeMeasurementKey;
                        currentPhasorGroup.PhaseB.Measurement.AngleKey = m_editedCurrentPhasor.BPhaseAngleMeasurementKey;
                        currentPhasorGroup.PhaseC.Measurement.MagnitudeKey = m_editedCurrentPhasor.CPhaseMagnitudeMeasurementKey;
                        currentPhasorGroup.PhaseC.Measurement.AngleKey = m_editedCurrentPhasor.CPhaseAngleMeasurementKey;
                        currentPhasorGroup.PositiveSequence.Measurement.MagnitudeKey = m_editedCurrentPhasor.PositiveSequenceMagnitudeEstimateKey;
                        currentPhasorGroup.PositiveSequence.Measurement.AngleKey = m_editedCurrentPhasor.PositiveSequenceAngleEstimateKey;
                        currentPhasorGroup.PhaseA.Measurement.MagnitudeKey = m_editedCurrentPhasor.APhaseMagnitudeEstimateKey;
                        currentPhasorGroup.PhaseA.Measurement.AngleKey = m_editedCurrentPhasor.APhaseAngleEstimateKey;
                        currentPhasorGroup.PhaseB.Measurement.MagnitudeKey = m_editedCurrentPhasor.BPhaseMagnitudeEstimateKey;
                        currentPhasorGroup.PhaseB.Measurement.AngleKey = m_editedCurrentPhasor.BPhaseAngleEstimateKey;
                        currentPhasorGroup.PhaseC.Measurement.MagnitudeKey = m_editedCurrentPhasor.CPhaseMagnitudeEstimateKey;
                        currentPhasorGroup.PhaseC.Measurement.AngleKey = m_editedCurrentPhasor.CPhaseAngleEstimateKey;
                    }
                }

                network.SerializeToXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);

                LoadCurrentPhasorView(LoadNetworkModel());
            }

            m_editedCurrentPhasor = null;
        }

        private void CurrentPhasorDataGrid_CellEditEnding(object sender, DataGridCellEditEndingEventArgs e)
        {
            if (e.EditAction == DataGridEditAction.Commit)
            {
                m_editedCurrentPhasor = e.Row.Item as CurrentPhasorDataModel;
            }
        }

        private void RefreshButton_Click(object sender, RoutedEventArgs e)
        {
            LoadCurrentPhasorView(LoadNetworkModel());
        }

        private Network LoadNetworkModel()
        {
            Network network = Network.DeserializeFromXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);
            network.Initialize();
            return network;
        }

        private void LoadCurrentPhasorView(Network network)
        {
            m_currentPhasorData.Clear();

            foreach (CurrentPhasorGroup currentPhasorGroup in network.Model.Currents)
            {
                m_currentPhasorData.Add(new CurrentPhasorDataModel()
                {
                    InternalID = currentPhasorGroup.InternalID,
                    Number = currentPhasorGroup.Number,
                    Acronym = currentPhasorGroup.Acronym,
                    Name = currentPhasorGroup.Name,
                    Description = currentPhasorGroup.Description,
                    IsEnabled = currentPhasorGroup.IsEnabled,
                    UseStatusFlag = currentPhasorGroup.UseStatusFlagForRemovingMeasurements,
                    PositiveSequenceVariance = currentPhasorGroup.PositiveSequence.Measurement.MeasurementVariance,
                    APhaseVariance = currentPhasorGroup.PhaseA.Measurement.MeasurementVariance,
                    BPhaseVariance = currentPhasorGroup.PhaseB.Measurement.MeasurementVariance,
                    CPhaseVariance = currentPhasorGroup.PhaseC.Measurement.MeasurementVariance,
                    InstrumentTransformerCalibrationSetting = currentPhasorGroup.PositiveSequence.Measurement.InstrumentTransformerCalibrationSetting,
                    PositiveSequenceRCF = currentPhasorGroup.PositiveSequence.Measurement.RCF,
                    APhaseRCF = currentPhasorGroup.PhaseA.Measurement.RCF,
                    BPhaseRCF = currentPhasorGroup.PhaseB.Measurement.RCF,
                    CPhaseRCF = currentPhasorGroup.PhaseC.Measurement.RCF,
                    PositiveSequencePACF = currentPhasorGroup.PositiveSequence.Measurement.PACF,
                    APhasePACF = currentPhasorGroup.PhaseA.Measurement.PACF,
                    BPhasePACF = currentPhasorGroup.PhaseB.Measurement.PACF,
                    CPhasePACF = currentPhasorGroup.PhaseC.Measurement.PACF,
                    PositiveSequenceMagnitudeMeasurementKey = currentPhasorGroup.PositiveSequence.Measurement.MagnitudeKey,
                    PositiveSequenceAngleMeasurementKey = currentPhasorGroup.PositiveSequence.Measurement.AngleKey,
                    APhaseMagnitudeMeasurementKey = currentPhasorGroup.PhaseA.Measurement.MagnitudeKey,
                    APhaseAngleMeasurementKey = currentPhasorGroup.PhaseA.Measurement.AngleKey,
                    BPhaseMagnitudeMeasurementKey = currentPhasorGroup.PhaseB.Measurement.MagnitudeKey,
                    BPhaseAngleMeasurementKey = currentPhasorGroup.PhaseB.Measurement.AngleKey,
                    CPhaseMagnitudeMeasurementKey = currentPhasorGroup.PhaseC.Measurement.MagnitudeKey,
                    CPhaseAngleMeasurementKey = currentPhasorGroup.PhaseC.Measurement.AngleKey,
                    PositiveSequenceMagnitudeEstimateKey = currentPhasorGroup.PositiveSequence.Estimate.MagnitudeKey,
                    PositiveSequenceAngleEstimateKey = currentPhasorGroup.PositiveSequence.Estimate.AngleKey,
                    APhaseMagnitudeEstimateKey = currentPhasorGroup.PhaseA.Estimate.MagnitudeKey,
                    APhaseAngleEstimateKey = currentPhasorGroup.PhaseA.Estimate.AngleKey,
                    BPhaseMagnitudeEstimateKey = currentPhasorGroup.PhaseB.Estimate.MagnitudeKey,
                    BPhaseAngleEstimateKey = currentPhasorGroup.PhaseB.Estimate.AngleKey,
                    CPhaseMagnitudeEstimateKey = currentPhasorGroup.PhaseC.Estimate.MagnitudeKey,
                    CPhaseAngleEstimateKey = currentPhasorGroup.PhaseC.Estimate.AngleKey,
                });
            }

        }
    }
}
