﻿//******************************************************************************************************
//  DivisionDataGridUserControl.cs
//
//  Copyright © 2013, Kevin D. Jones.  All Rights Reserved.
//
//  This file is licensed to you under the Eclipse Public License -v 1.0 (the "License"); you may
//  not use this file except in compliance with the License. You may obtain a copy of the License at:
//
//      http://www.opensource.org/licenses/eclipse-1.0.php
//
//  Unless agreed to in writing, the subject software distributed under the License is distributed on an
//  "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the
//  License for the specific language governing permissions and limitations.
//
//  Code Modification History:
//  ----------------------------------------------------------------------------------------------------
//  07/15/2013 - Kevin D. Jones
//       Generated original version of source code.
//
//******************************************************************************************************
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Configuration;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using NetworkModelBuilder.DataModels;
using FirstFloor.ModernUI.Windows.Controls;
using SynchrophasorAnalytics.Networks;
using SynchrophasorAnalytics.Modeling;
using SynchrophasorAnalytics.Measurements;

namespace NetworkModelBuilder.DataTables
{
    /// <summary>
    /// Interaction logic for DivisionDataGridUserControl.xaml
    /// </summary>
    public partial class DivisionDataGridUserControl : UserControl
    {
        private ObservableCollection<string> m_parentCompanies = new ObservableCollection<string>();
        private ObservableCollection<DivisionDataModel> m_divisionData;
        private DivisionDataModel m_editedDivision;

        public DivisionDataGridUserControl()
        {
            InitializeComponent();
            m_divisionData = new ObservableCollection<DivisionDataModel>();
            DivisionsDataGrid.CellEditEnding += DivisionsDataGrid_CellEditEnding;
            DivisionsDataGrid.CurrentCellChanged += DivisionsDataGrid_CurrentCellChanged;
            DivisionsDataGrid.DataContext = m_divisionData;
            ParentCompanyComboBox.ItemsSource = m_parentCompanies;
        }

        private void DivisionsDataGrid_CurrentCellChanged(object sender, EventArgs e)
        {
            if (m_editedDivision != null)
            {
                Network network = LoadNetworkModel();
                Company newParentCompany = null;
                Company oldParentCompany = null;
                Division editedDivision = null;

                foreach (Company company in network.Model.Companies)
                {
                    foreach (Division division in company.Divisions)
                    {
                        if (division.InternalID == m_editedDivision.InternalID)
                        {
                            division.Number = m_editedDivision.Number;
                            division.Name = m_editedDivision.Name;
                            division.Acronym = m_editedDivision.Acronym;
                            division.Description = m_editedDivision.Description;

                            if (!(m_editedDivision.ParentCompany.Equals(company.Name + " - ID: " + company.InternalID.ToString())))
                            {
                                foreach (Company newCompany in network.Model.Companies)
                                {
                                    if (m_editedDivision.ParentCompany.Equals(newCompany.Name + " - ID: " + newCompany.InternalID.ToString()))
                                    {
                                        newParentCompany = newCompany;
                                        oldParentCompany = company;
                                        editedDivision = division;
                                    }
                                }
                            }
                        }
                    }
                }

                if (newParentCompany != null && oldParentCompany != null && editedDivision != null)
                {
                    ModernDialog.ShowMessage("Division has changed owners", "Message", MessageBoxButton.OK);
                    newParentCompany.Divisions.Add(editedDivision);
                    oldParentCompany.Divisions.Remove(editedDivision);
                }

                network.SerializeToXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);

                LoadCurrentDivisionView(LoadNetworkModel());
            }

            m_editedDivision = null;
        }

        private void DivisionsDataGrid_CellEditEnding(object sender, DataGridCellEditEndingEventArgs e)
        {
            if (e.EditAction == DataGridEditAction.Commit)
            {
                m_editedDivision = e.Row.Item as DivisionDataModel;
            }
        }

        private void AddButton_Click(object sender, RoutedEventArgs e)
        {
            
        }

        private void RefreshButton_Click(object sender, RoutedEventArgs e)
        {
            LoadCurrentDivisionView(LoadNetworkModel());
        }

        private void DeleteButton_Click(object sender, RoutedEventArgs e)
        {
            DivisionDataModel selectedItem = (DivisionDataModel)DivisionsDataGrid.SelectedItem;

            if (selectedItem != null)
            {
                StringBuilder stringBuilder = new StringBuilder();
                stringBuilder.AppendFormat("Are you sure you want to delete the Division and all of its children Substations and Transmission Lines?{0}", Environment.NewLine);
                stringBuilder.AppendLine();
                stringBuilder.AppendFormat("InternalID: " + selectedItem.InternalID.ToString() + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Number: " + selectedItem.Number.ToString() + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Acronym: " + selectedItem.Acronym + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Name: " + selectedItem.Name + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Description: " + selectedItem.Description + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Parent Company: " + selectedItem.ParentCompany + "{0}", Environment.NewLine);

                MessageBoxButton confirmDeletionButtons = MessageBoxButton.YesNo;
                var confirmDeletionResult = ModernDialog.ShowMessage(stringBuilder.ToString(), "Confirm Deletion", confirmDeletionButtons);

                if (confirmDeletionResult.ToString().Equals("Yes"))
                {
                    Network network = LoadNetworkModel();

                    foreach (Company company in network.Model.Companies)
                    {
                        company.Divisions.RemoveAll(x => x.InternalID == selectedItem.InternalID);
                    }

                    network.SerializeToXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);

                    LoadCurrentDivisionView(LoadNetworkModel());
                }
            }
        }

        private void DivisionUserControl_Loaded(object sender, RoutedEventArgs e)
        {
            Network network = Network.DeserializeFromXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);
                network.Initialize();
            LoadCurrentDivisionView(network);

        }

        private void LoadCurrentDivisionView(Network network)
        {
            // Clear the current view
            m_divisionData.Clear();
            m_parentCompanies.Clear();

            foreach (Company company in network.Model.Companies)
            {
                // Clear the current view
                m_parentCompanies.Add(company.Name + " - ID: " + company.InternalID.ToString());

                foreach (Division division in company.Divisions)
                {
                    m_divisionData.Add(new DivisionDataModel(division.InternalID, division.Number, division.Acronym, division.Name, division.Description, company.Name + " - ID: " + company.InternalID.ToString()));
                }
            }
        }

        private Network LoadNetworkModel()
        {
            Network network = Network.DeserializeFromXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);
            network.Initialize();
            return network;
        }

        private void AddSubstationButton_Click(object sender, RoutedEventArgs e)
        {
            DivisionDataModel selectedItem = (DivisionDataModel)DivisionsDataGrid.SelectedItem;

            if (selectedItem == null)
            {
                ModernDialog.ShowMessage("Select a Division to add a Substation", "Add Substation Failed", MessageBoxButton.OK);
            }
            else
            {
                Network network = LoadNetworkModel();

                // Figure out which InternalID to use
                List<int> substationInternalIds = new List<int>();
                foreach (Company company in network.Model.Companies)
                {
                    foreach (Division division in company.Divisions)
                    {
                        foreach (Substation substation in division.Substations)
                        {
                            substationInternalIds.Add(substation.InternalID);
                        }
                    }
                }
                int newSubstationInternalID = substationInternalIds.Max() + 1;

                foreach (Company company in network.Model.Companies)
                {
                    foreach (Division division in company.Divisions)
                    {
                        if (division.InternalID == selectedItem.InternalID)
                        {
                            division.Substations.Add(new Substation()
                            {
                                InternalID = newSubstationInternalID,
                                Number = 1,
                                Acronym = "SUB",
                                Name = "Substation Name",
                                Description = "Substation Description",
                                ParentDivision = division,
                            });
                        }
                    }
                }
                network.SerializeToXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);
            }
        }

        private void AddTransmissionLineButton_Click(object sender, RoutedEventArgs e)
        {
            ModernDialog.ShowMessage("This button has not been implemented yet.", "Oh, Snap!", MessageBoxButton.OK);
        }
    }
}
