﻿//******************************************************************************************************
//  LineSegmentDataGridUserControl.cs
//
//  Copyright © 2013, Kevin D. Jones.  All Rights Reserved.
//
//  This file is licensed to you under the Eclipse Public License -v 1.0 (the "License"); you may
//  not use this file except in compliance with the License. You may obtain a copy of the License at:
//
//      http://www.opensource.org/licenses/eclipse-1.0.php
//
//  Unless agreed to in writing, the subject software distributed under the License is distributed on an
//  "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the
//  License for the specific language governing permissions and limitations.
//
//  Code Modification History:
//  ----------------------------------------------------------------------------------------------------
//  07/15/2013 - Kevin D. Jones
//       Generated original version of source code.
//
//******************************************************************************************************
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Configuration;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using NetworkModelBuilder.DataModels;
using FirstFloor.ModernUI.Windows.Controls;
using SynchrophasorAnalytics.Networks;
using SynchrophasorAnalytics.Modeling;

namespace NetworkModelBuilder.DataTables
{
    /// <summary>
    /// Interaction logic for LineSegmentDataGridUserControl.xaml
    /// </summary>
    public partial class LineSegmentDataGridUserControl : UserControl
    {
        private ObservableCollection<LineSegmentsDataModel> m_lineSegmentData;
        private List<string> m_parentTransmissionLineList = new List<string>();
        private List<string> m_fromNodeList = new List<string>();
        private List<string> m_toNodeList = new List<string>();

        public LineSegmentDataGridUserControl()
        {
            InitializeComponent();
            m_lineSegmentData = new ObservableCollection<LineSegmentsDataModel>();
            LineSegmentsDataGrid.DataContext = m_lineSegmentData;
            ParentTransmissionLineComboBox.ItemsSource = m_parentTransmissionLineList;
            FromNodeComboBox.ItemsSource = m_fromNodeList;
            ToNodeComboBox.ItemsSource = m_toNodeList;
            LoadLineSegmentView(LoadNetworkModel());
        }

        private void AddButton_Click(object sender, RoutedEventArgs e)
        {
            m_lineSegmentData.Add(new LineSegmentsDataModel());
        }

        private void RefreshButton_Click(object sender, RoutedEventArgs e)
        {

        }

        private void DeleteButton_Click(object sender, RoutedEventArgs e)
        {
            LineSegmentsDataModel selectedItem = (LineSegmentsDataModel)LineSegmentsDataGrid.SelectedItem;

            if (selectedItem != null)
            {
                StringBuilder stringBuilder = new StringBuilder();
                stringBuilder.AppendFormat("Are you sure you want to delete the Line Segment?{0}", Environment.NewLine);
                stringBuilder.AppendLine();
                stringBuilder.AppendFormat("InternalID: " + selectedItem.InternalID.ToString() + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Number: " + selectedItem.Number.ToString() + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Name: " + selectedItem.Name + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Description: " + selectedItem.Description + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("From Node: " + selectedItem.FromNode.ToString() + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("To Node: " + selectedItem.ToNode.ToString() + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Parent Transmission Line: " + selectedItem.ParentTransmissionLine + "{0}", Environment.NewLine);

                MessageBoxButton confirmDeletionButtons = MessageBoxButton.YesNo;
                var confirmDeletionResult = ModernDialog.ShowMessage(stringBuilder.ToString(), "Confirm Deletion", confirmDeletionButtons);

                if (confirmDeletionResult.ToString().Equals("Yes"))
                {
                    m_lineSegmentData.Remove((LineSegmentsDataModel)LineSegmentsDataGrid.SelectedItem);
                }
            }
        }

        private Network LoadNetworkModel()
        {
            Network network = Network.DeserializeFromXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);
            network.Initialize();
            return network;
        }

        private void LoadLineSegmentView(Network network)
        {
            m_lineSegmentData.Clear();
            m_fromNodeList.Clear();
            m_toNodeList.Clear();

            foreach (Company company in network.Model.Companies)
            {
                foreach (Division division in company.Divisions)
                {
                    foreach (TransmissionLine transmissionLine in division.TransmissionLines)
                    {
                        m_parentTransmissionLineList.Add(transmissionLine.ParentDivision.ParentCompany.Acronym + " - " + transmissionLine.Number.ToString() + " - " + transmissionLine.Description);
                        foreach (SynchrophasorAnalytics.Modeling.LineSegment lineSegment in transmissionLine.LineSegments)
                        {
                            if (lineSegment.FromNode.ParentSubstation != null && lineSegment.ToNode.ParentSubstation != null)
                            {
                                m_lineSegmentData.Add(new LineSegmentsDataModel()
                                {
                                    InternalID = lineSegment.InternalID,
                                    Number = lineSegment.Number,
                                    Name = lineSegment.Name,
                                    Description = lineSegment.Description,
                                    FromNode = lineSegment.FromNode.ParentSubstation.Name + " - " + lineSegment.FromNode.Name,
                                    ToNode = lineSegment.ToNode.ParentSubstation.Name + " - " + lineSegment.ToNode.Name,
                                    R1 = lineSegment.RawImpedanceParameters.R1,
                                    R2 = lineSegment.RawImpedanceParameters.R2,
                                    R3 = lineSegment.RawImpedanceParameters.R3,
                                    R4 = lineSegment.RawImpedanceParameters.R4,
                                    R5 = lineSegment.RawImpedanceParameters.R5,
                                    R6 = lineSegment.RawImpedanceParameters.R6,
                                    X1 = lineSegment.RawImpedanceParameters.X1,
                                    X2 = lineSegment.RawImpedanceParameters.X2,
                                    X3 = lineSegment.RawImpedanceParameters.X3,
                                    X4 = lineSegment.RawImpedanceParameters.X4,
                                    X5 = lineSegment.RawImpedanceParameters.X5,
                                    X6 = lineSegment.RawImpedanceParameters.X6,
                                    B1 = lineSegment.RawImpedanceParameters.B1,
                                    B2 = lineSegment.RawImpedanceParameters.B2,
                                    B3 = lineSegment.RawImpedanceParameters.B3,
                                    B4 = lineSegment.RawImpedanceParameters.B4,
                                    B5 = lineSegment.RawImpedanceParameters.B5,
                                    B6 = lineSegment.RawImpedanceParameters.B6,
                                    ParentTransmissionLine = transmissionLine.ParentDivision.ParentCompany.Acronym + " - " + lineSegment.ParentTransmissionLine.Number.ToString() + " - " + lineSegment.ParentTransmissionLine.Description
                                });
                            }
                            else if (lineSegment.FromNode.ParentSubstation != null && lineSegment.ToNode.ParentSubstation == null)
                            {
                                m_lineSegmentData.Add(new LineSegmentsDataModel()
                                {
                                    InternalID = lineSegment.InternalID,
                                    Number = lineSegment.Number,
                                    Name = lineSegment.Name,
                                    Description = lineSegment.Description,
                                    FromNode = lineSegment.FromNode.ParentSubstation.Name + " - " + lineSegment.FromNode.Name,
                                    ToNode = lineSegment.ToNode.ParentTransmissionLine.Name + " - " + lineSegment.ToNode.Name,
                                    R1 = lineSegment.RawImpedanceParameters.R1,
                                    R2 = lineSegment.RawImpedanceParameters.R2,
                                    R3 = lineSegment.RawImpedanceParameters.R3,
                                    R4 = lineSegment.RawImpedanceParameters.R4,
                                    R5 = lineSegment.RawImpedanceParameters.R5,
                                    R6 = lineSegment.RawImpedanceParameters.R6,
                                    X1 = lineSegment.RawImpedanceParameters.X1,
                                    X2 = lineSegment.RawImpedanceParameters.X2,
                                    X3 = lineSegment.RawImpedanceParameters.X3,
                                    X4 = lineSegment.RawImpedanceParameters.X4,
                                    X5 = lineSegment.RawImpedanceParameters.X5,
                                    X6 = lineSegment.RawImpedanceParameters.X6,
                                    B1 = lineSegment.RawImpedanceParameters.B1,
                                    B2 = lineSegment.RawImpedanceParameters.B2,
                                    B3 = lineSegment.RawImpedanceParameters.B3,
                                    B4 = lineSegment.RawImpedanceParameters.B4,
                                    B5 = lineSegment.RawImpedanceParameters.B5,
                                    B6 = lineSegment.RawImpedanceParameters.B6,
                                    ParentTransmissionLine = lineSegment.ParentTransmissionLine.Number.ToString() + " - " + lineSegment.ParentTransmissionLine.Description
                                });
                            }
                            else if (lineSegment.FromNode.ParentSubstation == null && lineSegment.ToNode.ParentSubstation != null)
                            {
                                m_lineSegmentData.Add(new LineSegmentsDataModel()
                                {
                                    InternalID = lineSegment.InternalID,
                                    Number = lineSegment.Number,
                                    Name = lineSegment.Name,
                                    Description = lineSegment.Description,
                                    FromNode = lineSegment.FromNode.ParentTransmissionLine.Name + " - " + lineSegment.FromNode.Name,
                                    ToNode = lineSegment.ToNode.ParentSubstation.Name + " - " + lineSegment.ToNode.Name,
                                    R1 = lineSegment.RawImpedanceParameters.R1,
                                    R2 = lineSegment.RawImpedanceParameters.R2,
                                    R3 = lineSegment.RawImpedanceParameters.R3,
                                    R4 = lineSegment.RawImpedanceParameters.R4,
                                    R5 = lineSegment.RawImpedanceParameters.R5,
                                    R6 = lineSegment.RawImpedanceParameters.R6,
                                    X1 = lineSegment.RawImpedanceParameters.X1,
                                    X2 = lineSegment.RawImpedanceParameters.X2,
                                    X3 = lineSegment.RawImpedanceParameters.X3,
                                    X4 = lineSegment.RawImpedanceParameters.X4,
                                    X5 = lineSegment.RawImpedanceParameters.X5,
                                    X6 = lineSegment.RawImpedanceParameters.X6,
                                    B1 = lineSegment.RawImpedanceParameters.B1,
                                    B2 = lineSegment.RawImpedanceParameters.B2,
                                    B3 = lineSegment.RawImpedanceParameters.B3,
                                    B4 = lineSegment.RawImpedanceParameters.B4,
                                    B5 = lineSegment.RawImpedanceParameters.B5,
                                    B6 = lineSegment.RawImpedanceParameters.B6,
                                    ParentTransmissionLine = lineSegment.ParentTransmissionLine.Number.ToString() + " - " + lineSegment.ParentTransmissionLine.Description
                                });
                            }
                            else if (lineSegment.FromNode.ParentSubstation == null && lineSegment.ToNode.ParentSubstation == null)
                            {
                                m_lineSegmentData.Add(new LineSegmentsDataModel()
                                {
                                    InternalID = lineSegment.InternalID,
                                    Number = lineSegment.Number,
                                    Name = lineSegment.Name,
                                    Description = lineSegment.Description,
                                    FromNode = lineSegment.FromNode.ParentTransmissionLine.Name + " - " + lineSegment.FromNode.Name,
                                    ToNode = lineSegment.ToNode.ParentTransmissionLine.Name + " - " + lineSegment.ToNode.Name,
                                    R1 = lineSegment.RawImpedanceParameters.R1,
                                    R2 = lineSegment.RawImpedanceParameters.R2,
                                    R3 = lineSegment.RawImpedanceParameters.R3,
                                    R4 = lineSegment.RawImpedanceParameters.R4,
                                    R5 = lineSegment.RawImpedanceParameters.R5,
                                    R6 = lineSegment.RawImpedanceParameters.R6,
                                    X1 = lineSegment.RawImpedanceParameters.X1,
                                    X2 = lineSegment.RawImpedanceParameters.X2,
                                    X3 = lineSegment.RawImpedanceParameters.X3,
                                    X4 = lineSegment.RawImpedanceParameters.X4,
                                    X5 = lineSegment.RawImpedanceParameters.X5,
                                    X6 = lineSegment.RawImpedanceParameters.X6,
                                    B1 = lineSegment.RawImpedanceParameters.B1,
                                    B2 = lineSegment.RawImpedanceParameters.B2,
                                    B3 = lineSegment.RawImpedanceParameters.B3,
                                    B4 = lineSegment.RawImpedanceParameters.B4,
                                    B5 = lineSegment.RawImpedanceParameters.B5,
                                    B6 = lineSegment.RawImpedanceParameters.B6,
                                    ParentTransmissionLine = lineSegment.ParentTransmissionLine.Number.ToString() + " - " + lineSegment.ParentTransmissionLine.Description
                                });
                            }
                        }

                        foreach (Node node in transmissionLine.Nodes)
                        {
                            if (node.ParentSubstation != null)
                            {
                                if (!m_fromNodeList.Contains(node.ParentSubstation.Name + " - " + node.Name))
                                {
                                    m_fromNodeList.Add(node.ParentSubstation.Name + " - " + node.Name);
                                }
                                if (!m_toNodeList.Contains(node.ParentSubstation.Name + " - " + node.Name))
                                {
                                    m_toNodeList.Add(node.ParentSubstation.Name + " - " + node.Name);
                                }
                            }
                            else
                            {
                                if (!m_fromNodeList.Contains(node.ParentTransmissionLine.Name + " - " + node.Name))
                                {
                                    m_fromNodeList.Add(node.ParentTransmissionLine.Name + " - " + node.Name);
                                }
                                if (!m_toNodeList.Contains(node.ParentTransmissionLine.Name + " - " + node.Name))
                                {
                                    m_toNodeList.Add(node.ParentTransmissionLine.Name + " - " + node.Name);
                                }
                            }
                        }

                        foreach (Node node in transmissionLine.FromSubstation.Nodes)
                        {
                            if (!m_fromNodeList.Contains(node.ParentSubstation.Name + " - " + node.Name))
                            {
                                m_fromNodeList.Add(node.ParentSubstation.Name + " - " + node.Name);
                            }
                        }

                        foreach (Node node in transmissionLine.ToSubstation.Nodes)
                        {
                            if (!m_toNodeList.Contains(node.ParentSubstation.Name + " - " + node.Name))
                            {
                                m_toNodeList.Add(node.ParentSubstation.Name + " - " + node.Name);
                            }
                        }
                    }
                }
            }
        }
    }
}
