﻿//******************************************************************************************************
//  StatusWordDataGridUserControl.cs
//
//  Copyright © 2013, Kevin D. Jones.  All Rights Reserved.
//
//  This file is licensed to you under the Eclipse Public License -v 1.0 (the "License"); you may
//  not use this file except in compliance with the License. You may obtain a copy of the License at:
//
//      http://www.opensource.org/licenses/eclipse-1.0.php
//
//  Unless agreed to in writing, the subject software distributed under the License is distributed on an
//  "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the
//  License for the specific language governing permissions and limitations.
//
//  Code Modification History:
//  ----------------------------------------------------------------------------------------------------
//  07/15/2013 - Kevin D. Jones
//       Generated original version of source code.
//
//******************************************************************************************************
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Configuration;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using NetworkModelBuilder.DataModels;
using FirstFloor.ModernUI.Windows.Controls;
using SynchrophasorAnalytics.Networks;
using SynchrophasorAnalytics.Measurements;

namespace NetworkModelBuilder.DataTables
{
    /// <summary>
    /// Interaction logic for StatusWordDataGridUserControl.xaml
    /// </summary>
    public partial class StatusWordDataGridUserControl : UserControl
    {
        private ObservableCollection<StatusWordDataModel> m_statusWordData;
        private StatusWordDataModel m_editedStatusWord;

        public StatusWordDataGridUserControl()
        {
            InitializeComponent();
            StatusWordDataGrid.CellEditEnding += StatusWordDataGrid_CellEditEnding;
            StatusWordDataGrid.CurrentCellChanged += StatusWordDataGrid_CurrentCellChanged;
            m_statusWordData = new ObservableCollection<StatusWordDataModel>();
            StatusWordDataGrid.DataContext = m_statusWordData;
            LoadStatusWordView(LoadNetworkModel());
        }

        private void StatusWordDataGrid_CurrentCellChanged(object sender, EventArgs e)
        {
            if (m_editedStatusWord != null)
            {
                Network network = LoadNetworkModel();

                foreach (StatusWord statusWord in network.Model.StatusWords)
                {
                    if (statusWord.InternalID == m_editedStatusWord.InternalID)
                    {
                        statusWord.Number = m_editedStatusWord.Number;
                        statusWord.Description = m_editedStatusWord.Description;
                        statusWord.Key = m_editedStatusWord.MeasurementKey;
                        statusWord.IsEnabled = m_editedStatusWord.IsEnabled;
                    }
                }

                network.SerializeToXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);

                LoadStatusWordView(LoadNetworkModel());
            }

            m_editedStatusWord = null;
        }

        private void StatusWordDataGrid_CellEditEnding(object sender, DataGridCellEditEndingEventArgs e)
        {
            if (e.EditAction == DataGridEditAction.Commit)
            {
                m_editedStatusWord = e.Row.Item as StatusWordDataModel;
            }
        }

        private void AddButton_Click(object sender, RoutedEventArgs e)
        {
            Network network = LoadNetworkModel();
            
            // Figure out which InternalID to use
            List<int> statusWordInternalIds = new List<int>();
            foreach (StatusWord statusWord in network.Model.StatusWords)
            {
                statusWordInternalIds.Add(statusWord.InternalID);
            }
            int newStatusWordInternalID = statusWordInternalIds.Max() + 1;

            network.Model.StatusWords.Add(new StatusWord()
            {
                InternalID = newStatusWordInternalID,
                Number = 1,
                Name = "Status Word Name",
                Description = "Status Word Description",
                IsEnabled = true,
                Key = "Undefined",
            });

            network.SerializeToXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);

            LoadStatusWordView(LoadNetworkModel());
        }

        private void RefreshButton_Click(object sender, RoutedEventArgs e)
        {
            LoadStatusWordView(LoadNetworkModel());
        }

        private void DeleteButton_Click(object sender, RoutedEventArgs e)
        {
            StatusWordDataModel selectedItem = (StatusWordDataModel)StatusWordDataGrid.SelectedItem;

            if (selectedItem != null)
            {
                StringBuilder stringBuilder = new StringBuilder();
                stringBuilder.AppendFormat("Are you sure you want to delete the Status Word?{0}", Environment.NewLine);
                stringBuilder.AppendLine();
                stringBuilder.AppendFormat("InternalID: " + selectedItem.InternalID.ToString() + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Number: " + selectedItem.Number.ToString() + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Description: " + selectedItem.Description + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Is Enabled: " + selectedItem.IsEnabled.ToString() + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Measurement Key: " + selectedItem.MeasurementKey + "{0}", Environment.NewLine);

                MessageBoxButton confirmDeletionButtons = MessageBoxButton.YesNo;
                var confirmDeletionResult = ModernDialog.ShowMessage(stringBuilder.ToString(), "Confirm Deletion", confirmDeletionButtons);

                if (confirmDeletionResult.ToString().Equals("Yes"))
                {
                    Network network = LoadNetworkModel();

                    network.Model.StatusWords.RemoveAll(x => x.InternalID == selectedItem.InternalID);

                    network.SerializeToXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);

                    LoadStatusWordView(LoadNetworkModel());
                }
            }
        }

        private void StatusWordUserControl_Loaded(object sender, RoutedEventArgs e)
        {

        }

        private Network LoadNetworkModel()
        {
            Network network = Network.DeserializeFromXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);
            network.Initialize();
            return network;
        }

        private void LoadStatusWordView(Network network)
        {
            m_statusWordData.Clear();

            foreach (StatusWord statusWord in network.Model.StatusWords)
            {
                m_statusWordData.Add(new StatusWordDataModel(statusWord.InternalID, statusWord.Number, statusWord.Description, statusWord.IsEnabled, statusWord.Key));
            }
        }
    }
}
