﻿//******************************************************************************************************
//  SubstationDataGridUserControl.cs
//
//  Copyright © 2013, Kevin D. Jones.  All Rights Reserved.
//
//  This file is licensed to you under the Eclipse Public License -v 1.0 (the "License"); you may
//  not use this file except in compliance with the License. You may obtain a copy of the License at:
//
//      http://www.opensource.org/licenses/eclipse-1.0.php
//
//  Unless agreed to in writing, the subject software distributed under the License is distributed on an
//  "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the
//  License for the specific language governing permissions and limitations.
//
//  Code Modification History:
//  ----------------------------------------------------------------------------------------------------
//  07/15/2013 - Kevin D. Jones
//       Generated original version of source code.
//
//******************************************************************************************************
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Configuration;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using NetworkModelBuilder.DataModels;
using FirstFloor.ModernUI.Windows.Controls;
using SynchrophasorAnalytics.Networks;
using SynchrophasorAnalytics.Modeling;
using SynchrophasorAnalytics.Measurements;

namespace NetworkModelBuilder.DataTables
{
    /// <summary>
    /// Interaction logic for SubstationDataGridUserControl.xaml
    /// </summary>
    public partial class SubstationDataGridUserControl : UserControl
    {
        private List<string> m_parentDivisions = new List<string>();

        private ObservableCollection<SubstationDataModel> m_substationData;
        private SubstationDataModel m_editedSubstation;

        public SubstationDataGridUserControl()
        {
            InitializeComponent();

            m_substationData = new ObservableCollection<SubstationDataModel>();
             
            SubstationDataGrid.DataContext = m_substationData;

            SubstationDataGrid.CellEditEnding += SubstationDataGrid_CellEditEnding;
            SubstationDataGrid.CurrentCellChanged += SubstationDataGrid_CurrentCellChanged;

            ParentDivisionComboBox.ItemsSource = m_parentDivisions;

            LoadSubstationView(LoadNetworkModel());
        }

        private void SubstationDataGrid_CurrentCellChanged(object sender, EventArgs e)
        {
            if (m_editedSubstation != null)
            {
                Network network = LoadNetworkModel();
                Division newParentDivision = null;
                Division oldParentDivision = null;
                Substation editedSubstation = null;

                foreach (Company company in network.Model.Companies)
                {
                    foreach (Division division in company.Divisions)
                    {
                        foreach (Substation substation in division.Substations)
                        {
                            if (substation.InternalID == m_editedSubstation.InternalID)
                            {
                                substation.Number = m_editedSubstation.Number;
                                substation.Name = m_editedSubstation.Name;
                                substation.Acronym = m_editedSubstation.Acronym;
                                substation.Description = m_editedSubstation.Description;

                                if (!(m_editedSubstation.ParentDivision.Equals(company.Acronym + " - " + division.Name + " - ID: " + company.InternalID.ToString())))
                                {
                                    foreach (Company newCompany in network.Model.Companies)
                                    {
                                        foreach (Division newDivision in newCompany.Divisions)
                                        {
                                            if (m_editedSubstation.ParentDivision.Equals(newCompany.Acronym + " - " + newDivision.Name + " - ID: " + newDivision.InternalID.ToString()))
                                            {
                                                newParentDivision = newDivision;
                                                oldParentDivision = division;
                                                editedSubstation = substation;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                if (newParentDivision != null && oldParentDivision != null && editedSubstation != null)
                {
                    ModernDialog.ShowMessage("Substation has changed owners", "Message", MessageBoxButton.OK);
                    newParentDivision.Substations.Add(editedSubstation);
                    oldParentDivision.Substations.Remove(editedSubstation);
                }

                network.SerializeToXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);

                LoadSubstationView(LoadNetworkModel());
            }

            m_editedSubstation = null;
        }

        private void SubstationDataGrid_CellEditEnding(object sender, DataGridCellEditEndingEventArgs e)
        {
            if (e.EditAction == DataGridEditAction.Commit)
            {
                m_editedSubstation = e.Row.Item as SubstationDataModel;
            }
        }

        private void AddButton_Click(object sender, RoutedEventArgs e)
        {
        }

        private void RefreshButton_Click(object sender, RoutedEventArgs e)
        {
            LoadSubstationView(LoadNetworkModel());
        }

        private void DeleteButton_Click(object sender, RoutedEventArgs e)
        {
            SubstationDataModel selectedItem = (SubstationDataModel)SubstationDataGrid.SelectedItem;

            if (selectedItem != null)
            {
                StringBuilder stringBuilder = new StringBuilder();
                stringBuilder.AppendFormat("Are you sure you want to delete the Substation?{0}", Environment.NewLine);
                stringBuilder.AppendLine();
                stringBuilder.AppendFormat("InternalID: " + selectedItem.InternalID.ToString() + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Number: " + selectedItem.Number.ToString() + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Acronym: " + selectedItem.Acronym + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Name: " + selectedItem.Name + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Description: " + selectedItem.Description + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Parent Division: " + selectedItem.ParentDivision + "{0}", Environment.NewLine);

                MessageBoxButton confirmDeletionButtons = MessageBoxButton.YesNo;
                var confirmDeletionResult = ModernDialog.ShowMessage(stringBuilder.ToString(), "Confirm Deletion", confirmDeletionButtons);

                if (confirmDeletionResult.ToString().Equals("Yes"))
                {
                    Network network = LoadNetworkModel();

                    int companyIndex = 0;
                    int divisionIndex = 0;

                    foreach (Company company in network.Model.Companies)
                    {
                        foreach (Division division in company.Divisions)
                        {
                            foreach (Substation substation in division.Substations)
                            {
                                if (substation.InternalID == selectedItem.InternalID)
                                {
                                    companyIndex = network.Model.Companies.IndexOf(company);
                                    divisionIndex = company.Divisions.IndexOf(division);
                                }
                            }
                        }
                    }

                    network.Model.Companies[companyIndex].Divisions[divisionIndex].Substations.RemoveAll(x => x.InternalID == selectedItem.InternalID);
                    
                    network.SerializeToXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);

                    LoadSubstationView(LoadNetworkModel());
                }
            }
        }

        private void SubstationUserControl_Loaded(object sender, RoutedEventArgs e)
        {
        }

        private Network LoadNetworkModel()
        {
            Network network = Network.DeserializeFromXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);
            network.Initialize();
            return network;
        }

        private void LoadSubstationView(Network network)
        {
            // Clear the current view;
            m_substationData.Clear();
            m_parentDivisions.Clear();

            foreach (Company company in network.Model.Companies)
            {
                foreach (Division division in company.Divisions)
                {
                    m_parentDivisions.Add(division.ParentCompany.Acronym + " - " + division.Name + " - ID: " + division.InternalID.ToString());
                    foreach (Substation substation in division.Substations)
                    {
                        m_substationData.Add(new SubstationDataModel(substation.InternalID, substation.Number, substation.Acronym, substation.Name, substation.Description, substation.ParentDivision.ParentCompany.Acronym + " - " + substation.ParentDivision.Name + " - ID: " + division.InternalID.ToString()));
                    }
                }
            }
        }

        private void AddNodeButton_Click(object sender, RoutedEventArgs e)
        {
            Network network = LoadNetworkModel();

            // Figure out which internal ID to use
            List<int> nodeInternalIDs = new List<int>();
            foreach (Company company in network.Model.Companies)
            {
                foreach (Division division in company.Divisions)
                {
                    foreach (Substation substation in division.Substations)
                    {
                        foreach (Node node in substation.Nodes)
                        {
                            nodeInternalIDs.Add(node.InternalID);
                        }
                    }
                }
            }

            int newNodeInternalID = nodeInternalIDs.Max() + 1;

            SubstationDataModel selectedItem = (SubstationDataModel)SubstationDataGrid.SelectedItem;

            if (selectedItem == null)
            {
                ModernDialog.ShowMessage("Select a Substation to add a Node", "Add Node Failed", MessageBoxButton.OK);
            }
            else
            {
                foreach (Company company in network.Model.Companies)
                {
                    foreach (Division division in company.Divisions)
                    {
                        foreach (Substation substation in division.Substations)
                        {
                            if (substation.InternalID == selectedItem.InternalID)
                            {
                                substation.Nodes.Add(new Node()
                                {
                                    InternalID = newNodeInternalID,
                                    Number = 1,
                                    Name = "Node Name",
                                    Description = "Node Description",
                                    BaseKV = network.Model.VoltageLevels[0],
                                    ParentSubstation = substation,
                                    Voltage = new VoltagePhasorGroup(newNodeInternalID, 1, "Voltage Phasor Group Name", "Voltage Phasor Group Description")
                                });
                            }
                        }
                    }
                }
            }

            network.SerializeToXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);
        }

        private void AddTransformerButton_Click(object sender, RoutedEventArgs e)
        {
            ModernDialog.ShowMessage("This button has not been implemented yet.", "Oh, Snap!", MessageBoxButton.OK);
        }

        private void AddCircuitBreakerButton_Click(object sender, RoutedEventArgs e)
        {
            Network network = LoadNetworkModel();

            // Figure out which internal ID to use
            List<int> circuitBreakerInternalIDs = new List<int>();
            foreach (Company company in network.Model.Companies)
            {
                foreach (Division division in company.Divisions)
                {
                    foreach (Substation substation in division.Substations)
                    {
                        foreach (CircuitBreaker circuitBreaker in substation.CircuitBreakers)
                        {
                            circuitBreakerInternalIDs.Add(circuitBreaker.InternalID);
                        }
                    }
                }
            }

            int newCircuitBreakerInternalID = circuitBreakerInternalIDs.Max() + 1;

            SubstationDataModel selectedItem = (SubstationDataModel)SubstationDataGrid.SelectedItem;

            if (selectedItem == null)
            {
                ModernDialog.ShowMessage("Select a Substation to add a Circuit Breaker", "Add Circuit Breaker Failed", MessageBoxButton.OK);
            }
            else
            {
                foreach (Company company in network.Model.Companies)
                {
                    foreach (Division division in company.Divisions)
                    {
                        foreach (Substation substation in division.Substations)
                        {
                            if (substation.InternalID == selectedItem.InternalID)
                            {
                                substation.CircuitBreakers.Add(new CircuitBreaker()
                                {
                                    InternalID = newCircuitBreakerInternalID,
                                    Number = 1,
                                    Name = "Circuit Breaker Name",
                                    Description = "Circuit Breaker Description",
                                    MeasurementKey = "Undefined",
                                    NormalState = SwitchingDeviceNormalState.Closed,
                                    StatusID = 1,
                                    ParentSubstation = substation,
                                });
                            }
                        }
                    }
                }
            }

            network.SerializeToXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);

        }

        private void AddSwitchButton_Click(object sender, RoutedEventArgs e)
        {
            ModernDialog.ShowMessage("This button has not been implemented yet.", "Oh, Snap!", MessageBoxButton.OK);
        }
    }
}
