﻿//******************************************************************************************************
//  SwitchDataGridUserControl.cs
//
//  Copyright © 2013, Kevin D. Jones.  All Rights Reserved.
//
//  This file is licensed to you under the Eclipse Public License -v 1.0 (the "License"); you may
//  not use this file except in compliance with the License. You may obtain a copy of the License at:
//
//      http://www.opensource.org/licenses/eclipse-1.0.php
//
//  Unless agreed to in writing, the subject software distributed under the License is distributed on an
//  "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the
//  License for the specific language governing permissions and limitations.
//
//  Code Modification History:
//  ----------------------------------------------------------------------------------------------------
//  07/15/2013 - Kevin D. Jones
//       Generated original version of source code.
//
//******************************************************************************************************
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using NetworkModelBuilder.DataModels;
using FirstFloor.ModernUI.Windows.Controls;

namespace NetworkModelBuilder.DataTables
{
    /// <summary>
    /// Interaction logic for SwitchDataGridUserControl.xaml
    /// </summary>
    public partial class SwitchDataGridUserControl : UserControl
    {
        private ObservableCollection<SwitchDataModel> m_switchData;

        public SwitchDataGridUserControl()
        {
            InitializeComponent();
            m_switchData = new ObservableCollection<SwitchDataModel>();
            SwitchDataGrid.DataContext = m_switchData;
        }

        private void AddButton_Click(object sender, RoutedEventArgs e)
        {
            m_switchData.Add(new SwitchDataModel());
        }

        private void RefreshButton_Click(object sender, RoutedEventArgs e)
        {

        }

        private void DeleteButton_Click(object sender, RoutedEventArgs e)
        {
            SwitchDataModel selectedItem = (SwitchDataModel)SwitchDataGrid.SelectedItem;

            if (selectedItem != null)
            {
                StringBuilder stringBuilder = new StringBuilder();
                stringBuilder.AppendFormat("Are you sure you want to delete the Switch?{0}", Environment.NewLine);
                stringBuilder.AppendLine();
                stringBuilder.AppendFormat("InternalID: " + selectedItem.InternalID.ToString() + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Number: " + selectedItem.Number.ToString() + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Name: " + selectedItem.Name + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Description: " + selectedItem.Description + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Key: " + selectedItem.Key + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Normal State: " + selectedItem.NormalState.ToString() + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Parent Substation: " + selectedItem.ParentSubstation + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Parent Transmission Line: " + selectedItem.ParentTransmissionLine + "{0}", Environment.NewLine);

                MessageBoxButton confirmDeletionButtons = MessageBoxButton.YesNo;
                var confirmDeletionResult = ModernDialog.ShowMessage(stringBuilder.ToString(), "Confirm Deletion", confirmDeletionButtons);

                if (confirmDeletionResult.ToString().Equals("Yes"))
                {
                    m_switchData.Remove((SwitchDataModel)SwitchDataGrid.SelectedItem);
                }
            }
        }
    }
}
