﻿//******************************************************************************************************
//  TapConfigurationDataGridUserControl.cs
//
//  Copyright © 2013, Kevin D. Jones.  All Rights Reserved.
//
//  This file is licensed to you under the Eclipse Public License -v 1.0 (the "License"); you may
//  not use this file except in compliance with the License. You may obtain a copy of the License at:
//
//      http://www.opensource.org/licenses/eclipse-1.0.php
//
//  Unless agreed to in writing, the subject software distributed under the License is distributed on an
//  "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the
//  License for the specific language governing permissions and limitations.
//
//  Code Modification History:
//  ----------------------------------------------------------------------------------------------------
//  07/15/2013 - Kevin D. Jones
//       Generated original version of source code.
//
//******************************************************************************************************
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Configuration;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using NetworkModelBuilder.DataModels;
using FirstFloor.ModernUI.Windows.Controls;
using SynchrophasorAnalytics.Networks;
using SynchrophasorAnalytics.Modeling;

namespace NetworkModelBuilder.DataTables
{
    /// <summary>
    /// Interaction logic for TapConfigurationDataGridUserControl.xaml
    /// </summary>
    public partial class TapConfigurationDataGridUserControl : UserControl
    {
        #region [ Private Members ]

        private ObservableCollection<TapConfigurationDataModel> m_tapConfigurationData;
        private TapConfigurationDataModel m_editedTapConfiguration;

        #endregion

        #region [ Constructors ]

        public TapConfigurationDataGridUserControl()
        {
            InitializeComponent();
            m_tapConfigurationData = new ObservableCollection<TapConfigurationDataModel>();
            TapConfigurationDataGrid.DataContext = m_tapConfigurationData;
            TapConfigurationDataGrid.CellEditEnding += TapConfigurationDataGrid_CellEditEnding;
            TapConfigurationDataGrid.CurrentCellChanged += TapConfigurationDataGrid_CurrentCellChanged;
            LoadTapConfigurationView(LoadNetworkModel());
        }

        #endregion

        #region [ Editing Event Handlers ]

        private void TapConfigurationDataGrid_CurrentCellChanged(object sender, EventArgs e)
        {
            if (m_editedTapConfiguration != null)
            {
                Network network = LoadNetworkModel();

                foreach (TapConfiguration tapConfiguration in network.Model.TapConfigurations)
                {
                    if (tapConfiguration.InternalID == m_editedTapConfiguration.InternalID)
                    {
                        tapConfiguration.Acronym = m_editedTapConfiguration.Acronym;
                        tapConfiguration.Name = m_editedTapConfiguration.Name;
                        tapConfiguration.Description = m_editedTapConfiguration.Description;
                        tapConfiguration.LowerBounds = m_editedTapConfiguration.LowerBounds;
                        tapConfiguration.UpperBounds = m_editedTapConfiguration.UpperBounds;
                        tapConfiguration.PositionLowerBounds = m_editedTapConfiguration.PositionLowerBounds;
                        tapConfiguration.PositionUpperBounds = m_editedTapConfiguration.PositionUpperBounds;
                        tapConfiguration.PositionNominal = m_editedTapConfiguration.PositionNominal;
                    }
                }

                network.SerializeToXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);

                LoadTapConfigurationView(LoadNetworkModel());
            }

            m_editedTapConfiguration = null;
        }

        private void TapConfigurationDataGrid_CellEditEnding(object sender, DataGridCellEditEndingEventArgs e)
        {
            if (e.EditAction == DataGridEditAction.Commit)
            {
                m_editedTapConfiguration = e.Row.Item as TapConfigurationDataModel;
            }
        }

        #endregion

        #region [ Button Click Event Handlers ]

        private void AddButton_Click(object sender, RoutedEventArgs e)
        {
            Network network = LoadNetworkModel();

            // Figure out which InternalID to use
            List<int> tapConfigurationInternalIds = new List<int>();
            foreach (TapConfiguration tapConfiguration in network.Model.TapConfigurations)
            {
                tapConfigurationInternalIds.Add(tapConfiguration.InternalID);
            }
            int newTapConfigurationInternalID = tapConfigurationInternalIds.Max() + 1;

            network.Model.TapConfigurations.Add(new TapConfiguration()
            {
                InternalID = newTapConfigurationInternalID,
                Acronym = "TAP",
                Name = "Tap Configuration Name",
                Description = "Tap Configuration Description",
                LowerBounds = 0.92,
                UpperBounds = 1.08,
                PositionLowerBounds = -16,
                PositionUpperBounds = 16,
                PositionNominal = 0
            });

            network.SerializeToXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);

            LoadTapConfigurationView(LoadNetworkModel());
        }

        private void RefreshButton_Click(object sender, RoutedEventArgs e)
        {
            LoadTapConfigurationView(LoadNetworkModel());
        }

        private void DeleteButton_Click(object sender, RoutedEventArgs e)
        {
            TapConfigurationDataModel selectedItem = (TapConfigurationDataModel)TapConfigurationDataGrid.SelectedItem;

            if (selectedItem != null)
            {
                StringBuilder stringBuilder = new StringBuilder();
                stringBuilder.AppendFormat("Are you sure you want to delete the Tap Configuration?{0}", Environment.NewLine);
                stringBuilder.AppendLine();
                stringBuilder.AppendFormat("InternalID: " + selectedItem.InternalID.ToString() + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Name: " + selectedItem.Name + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Description: " + selectedItem.Description + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Lower Bounds: " + selectedItem.LowerBounds.ToString() + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Upper Bounds: " + selectedItem.UpperBounds.ToString() + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Lowest Position: " + selectedItem.PositionLowerBounds.ToString() + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Highest Position: " + selectedItem.PositionUpperBounds.ToString() + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Nominal Position: " + selectedItem.PositionNominal.ToString() + "{0}", Environment.NewLine);

                MessageBoxButton confirmDeletionButtons = MessageBoxButton.YesNo;
                var confirmDeletionResult = ModernDialog.ShowMessage(stringBuilder.ToString(), "Confirm Deletion", confirmDeletionButtons);

                if (confirmDeletionResult.ToString().Equals("Yes"))
                {
                    Network network = LoadNetworkModel();

                    network.Model.TapConfigurations.RemoveAll(x => x.InternalID == selectedItem.InternalID);

                    network.SerializeToXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);

                    LoadTapConfigurationView(LoadNetworkModel());
                }
            }
        }

        #endregion

        #region [ Private Methods ]

        private Network LoadNetworkModel()
        {
            Network network = Network.DeserializeFromXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);
            network.Initialize();
            return network;
        }

        private void LoadTapConfigurationView(Network network)
        {
            m_tapConfigurationData.Clear();

            foreach (TapConfiguration tapConfiguration in network.Model.TapConfigurations)
            {
                m_tapConfigurationData.Add(new TapConfigurationDataModel()
                {
                    InternalID = tapConfiguration.InternalID,
                    Acronym = tapConfiguration.Acronym,
                    Name = tapConfiguration.Name,
                    Description = tapConfiguration.Description,
                    LowerBounds = tapConfiguration.LowerBounds,
                    UpperBounds = tapConfiguration.UpperBounds,
                    PositionLowerBounds = tapConfiguration.PositionLowerBounds,
                    PositionUpperBounds = tapConfiguration.PositionUpperBounds,
                    PositionNominal = tapConfiguration.PositionNominal
                });
            }
        }

        #endregion
    }
}
