﻿//******************************************************************************************************
//  TransmissionLineDataGridUserControl.cs
//
//  Copyright © 2013, Kevin D. Jones.  All Rights Reserved.
//
//  This file is licensed to you under the Eclipse Public License -v 1.0 (the "License"); you may
//  not use this file except in compliance with the License. You may obtain a copy of the License at:
//
//      http://www.opensource.org/licenses/eclipse-1.0.php
//
//  Unless agreed to in writing, the subject software distributed under the License is distributed on an
//  "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the
//  License for the specific language governing permissions and limitations.
//
//  Code Modification History:
//  ----------------------------------------------------------------------------------------------------
//  07/15/2013 - Kevin D. Jones
//       Generated original version of source code.
//
//******************************************************************************************************
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Configuration;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using NetworkModelBuilder.DataModels;
using FirstFloor.ModernUI.Windows.Controls;
using SynchrophasorAnalytics.Networks;
using SynchrophasorAnalytics.Modeling;

namespace NetworkModelBuilder.DataTables
{
    /// <summary>
    /// Interaction logic for TransmissionLineDataGridUserControl.xaml
    /// </summary>
    public partial class TransmissionLineDataGridUserControl : UserControl
    {
        private ObservableCollection<TransmissionLineDataModel> m_transmissionLineData;
        private List<string> m_parentDivisionsList = new List<string>();
        private List<string> m_substationsList = new List<string>();

        public TransmissionLineDataGridUserControl()
        {
            InitializeComponent();
            m_transmissionLineData = new ObservableCollection<TransmissionLineDataModel>();
            TransmissionLineDataGrid.DataContext = m_transmissionLineData;
        }

        private void AddButton_Click(object sender, RoutedEventArgs e)
        {
            m_transmissionLineData.Add(new TransmissionLineDataModel());
        }

        private void RefreshButton_Click(object sender, RoutedEventArgs e)
        {

        }

        private void DeleteButton_Click(object sender, RoutedEventArgs e)
        {
            TransmissionLineDataModel selectedItem = (TransmissionLineDataModel)TransmissionLineDataGrid.SelectedItem;

            if (selectedItem != null)
            {
                StringBuilder stringBuilder = new StringBuilder();
                stringBuilder.AppendFormat("Are you sure you want to delete the Transmission Line?{0}", Environment.NewLine);
                stringBuilder.AppendLine();
                stringBuilder.AppendFormat("InternalID: " + selectedItem.ToString() + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Value: " + selectedItem.ToString() + "{0}", Environment.NewLine);

                MessageBoxButton confirmDeletionButtons = MessageBoxButton.YesNo;
                var confirmDeletionResult = ModernDialog.ShowMessage(stringBuilder.ToString(), "Confirm Deletion", confirmDeletionButtons);

                if (confirmDeletionResult.ToString().Equals("Yes"))
                {
                    m_transmissionLineData.Remove((TransmissionLineDataModel)TransmissionLineDataGrid.SelectedItem);
                }
            }
        }

        private void TransmissionLineUserControl_Loaded(object sender, RoutedEventArgs e)
        {
            Network network = Network.DeserializeFromXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);
            network.Initialize();
            LoadTransmissionLineView(network);
        }

        private void LoadTransmissionLineView(Network network)
        {
            m_transmissionLineData.Clear();
            m_parentDivisionsList.Clear();
            m_substationsList.Clear();

            foreach (Company company in network.Model.Companies)
            {
                foreach (Division division in company.Divisions)
                {
                    m_parentDivisionsList.Add(division.ParentCompany.Acronym + " - " + division.Name);
                    foreach (TransmissionLine transmissionLine in division.TransmissionLines)
                    {
                        m_transmissionLineData.Add(new TransmissionLineDataModel()
                        {
                            InternalID = transmissionLine.InternalID,
                            Number = transmissionLine.Number,
                            Acronym = transmissionLine.Acronym,
                            Name = transmissionLine.Name,
                            Description = transmissionLine.Description,
                            FromSubstation = transmissionLine.ParentDivision.ParentCompany.Acronym + " - " + transmissionLine.ParentDivision.Acronym + " - " + transmissionLine.FromSubstation.Name,
                            ToSubstation = transmissionLine.ParentDivision.ParentCompany.Acronym + " - " + transmissionLine.ParentDivision.Acronym + " - " + transmissionLine.ToSubstation.Name,
                            FromNode = transmissionLine.FromNodeID,
                            ToNode = transmissionLine.ToNodeID,
                            FromCurrent = transmissionLine.FromSubstationCurrent.Name,
                            ToCurrent = transmissionLine.ToSubstationCurrent.Name,
                            ParentDivision = transmissionLine.ParentDivision.ParentCompany.Acronym + " - " + transmissionLine.ParentDivision.Name
                        });
                    }

                    foreach (Substation substation in division.Substations)
                    {
                        m_substationsList.Add(substation.ParentDivision.ParentCompany.Acronym + " - " + substation.Name);
                    }
                }
            }
        }
    }
}
