﻿//******************************************************************************************************
//  VoltageLevelDataGridUserControl.cs
//
//  Copyright © 2013, Kevin D. Jones.  All Rights Reserved.
//
//  This file is licensed to you under the Eclipse Public License -v 1.0 (the "License"); you may
//  not use this file except in compliance with the License. You may obtain a copy of the License at:
//
//      http://www.opensource.org/licenses/eclipse-1.0.php
//
//  Unless agreed to in writing, the subject software distributed under the License is distributed on an
//  "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the
//  License for the specific language governing permissions and limitations.
//
//  Code Modification History:
//  ----------------------------------------------------------------------------------------------------
//  07/15/2013 - Kevin D. Jones
//       Generated original version of source code.
//
//******************************************************************************************************
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Configuration;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using NetworkModelBuilder.DataModels;
using FirstFloor.ModernUI.Windows.Controls;
using SynchrophasorAnalytics.Networks;
using SynchrophasorAnalytics.Modeling;

namespace NetworkModelBuilder.DataTables
{
    /// <summary>
    /// Interaction logic for VoltageLevelDataGridUserControl.xaml
    /// </summary>
    public partial class VoltageLevelDataGridUserControl : UserControl
    {
        private ObservableCollection<VoltageLevelDataModel> m_voltageLevelData;
        private VoltageLevelDataModel m_editedVoltageLevel;

        public VoltageLevelDataGridUserControl()
        {
            InitializeComponent();
            VoltageLevelDataGrid.CellEditEnding += VoltageLevelDataGrid_CellEditEnding;
            VoltageLevelDataGrid.CurrentCellChanged += VoltageLevelDataGrid_CurrentCellChanged;
            m_voltageLevelData = new ObservableCollection<VoltageLevelDataModel>();
            VoltageLevelDataGrid.DataContext = m_voltageLevelData;
            LoadCurrentVoltageLevelView(LoadNetworkModel());

        }

        private void VoltageLevelDataGrid_CurrentCellChanged(object sender, EventArgs e)
        {
            if (m_editedVoltageLevel != null)
            {
                Network network = LoadNetworkModel();

                foreach (VoltageLevel voltageLevel in network.Model.VoltageLevels)
                {
                    if (voltageLevel.InternalID == m_editedVoltageLevel.InternalID)
                    {
                        voltageLevel.Value = m_editedVoltageLevel.BaseKV;
                    }
                }

                network.SerializeToXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);

                LoadCurrentVoltageLevelView(LoadNetworkModel());
            }

            m_editedVoltageLevel = null;
        }

        private void VoltageLevelDataGrid_CellEditEnding(object sender, DataGridCellEditEndingEventArgs e)
        {
            if (e.EditAction == DataGridEditAction.Commit)
            {
                m_editedVoltageLevel = e.Row.Item as VoltageLevelDataModel;
            }
        }

        private void AddButton_Click(object sender, RoutedEventArgs e)
        {
            Network network = LoadNetworkModel();

            // Figure out which InternalID to use
            List<int> voltageLevelInternalIds = new List<int>();
            foreach (VoltageLevel voltageLevel in network.Model.VoltageLevels)
            {
                voltageLevelInternalIds.Add(voltageLevel.InternalID);
            }
            int newVoltageLevelInternalID = voltageLevelInternalIds.Max() + 1;

            network.Model.VoltageLevels.Add(new VoltageLevel()
            {
                InternalID = newVoltageLevelInternalID,
                Value = 500,
       
            });

            network.SerializeToXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);

            LoadCurrentVoltageLevelView(LoadNetworkModel());
        }

        private void RefreshButton_Click(object sender, RoutedEventArgs e)
        {
            LoadCurrentVoltageLevelView(LoadNetworkModel());
        }

        private void DeleteButton_Click(object sender, RoutedEventArgs e)
        {
            VoltageLevelDataModel selectedItem = (VoltageLevelDataModel)VoltageLevelDataGrid.SelectedItem;

            if (selectedItem != null)
            {
                StringBuilder stringBuilder = new StringBuilder();
                stringBuilder.AppendFormat("Are you sure you want to delete the VoltageLevel record?{0}", Environment.NewLine);
                stringBuilder.AppendLine();
                stringBuilder.AppendFormat("InternalID: " + selectedItem.InternalID.ToString() + "{0}", Environment.NewLine);
                stringBuilder.AppendFormat("Value: " + selectedItem.BaseKV.ToString() + "{0}", Environment.NewLine);

                MessageBoxButton confirmDeletionButtons = MessageBoxButton.YesNo;
                var confirmDeletionResult = ModernDialog.ShowMessage(stringBuilder.ToString(), "Confirm Deletion", confirmDeletionButtons);

                if (confirmDeletionResult.ToString().Equals("Yes"))
                {
                    Network network = LoadNetworkModel();

                    network.Model.VoltageLevels.RemoveAll(x => x.InternalID == selectedItem.InternalID);

                    network.SerializeToXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);

                    LoadCurrentVoltageLevelView(LoadNetworkModel());
                }
            }

        }

        private void VoltageLevelUserControl_Loaded(object sender, RoutedEventArgs e)
        {

        }

        private Network LoadNetworkModel()
        {
            Network network = Network.DeserializeFromXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);
            network.Initialize();
            return network;
        }

        private void LoadCurrentVoltageLevelView(Network network)
        {
            // Clear the current view
            m_voltageLevelData.Clear();

            foreach (VoltageLevel voltageLevel in network.Model.VoltageLevels)
            {
                m_voltageLevelData.Add(new VoltageLevelDataModel(voltageLevel.InternalID, voltageLevel.Value));
            }
        }

    }
}
