﻿//******************************************************************************************************
//  VoltagePhasorDataGridUserControl.cs
//
//  Copyright © 2013, Kevin D. Jones.  All Rights Reserved.
//
//  This file is licensed to you under the Eclipse Public License -v 1.0 (the "License"); you may
//  not use this file except in compliance with the License. You may obtain a copy of the License at:
//
//      http://www.opensource.org/licenses/eclipse-1.0.php
//
//  Unless agreed to in writing, the subject software distributed under the License is distributed on an
//  "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the
//  License for the specific language governing permissions and limitations.
//
//  Code Modification History:
//  ----------------------------------------------------------------------------------------------------
//  07/15/2013 - Kevin D. Jones
//       Generated original version of source code.
//
//******************************************************************************************************
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Configuration;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using NetworkModelBuilder.DataModels;
using FirstFloor.ModernUI.Windows.Controls;
using SynchrophasorAnalytics.Networks;
using SynchrophasorAnalytics.Modeling;
using SynchrophasorAnalytics.Measurements;

namespace NetworkModelBuilder.DataTables
{
    /// <summary>
    /// Interaction logic for VoltagePhasorDataGridUserControl.xaml
    /// </summary>
    public partial class VoltagePhasorDataGridUserControl : UserControl
    {
        private ObservableCollection<VoltagePhasorDataModel> m_voltagePhasorData;
        private VoltagePhasorDataModel m_editedVoltagePhasor;

        public VoltagePhasorDataGridUserControl()
        {
            InitializeComponent();
            VoltagePhasorDataGrid.CellEditEnding += VoltagePhasorDataGrid_CellEditEnding;
            VoltagePhasorDataGrid.CurrentCellChanged += VoltagePhasorDataGrid_CurrentCellChanged;
            m_voltagePhasorData = new ObservableCollection<VoltagePhasorDataModel>();
            VoltagePhasorDataGrid.DataContext = m_voltagePhasorData;
            LoadVoltagePhasorView(LoadNetworkModel());
        }

        private void VoltagePhasorDataGrid_CurrentCellChanged(object sender, EventArgs e)
        {
            if (m_editedVoltagePhasor != null)
            {
                Network network = LoadNetworkModel();

                foreach (VoltagePhasorGroup voltagePhasorGroup in network.Model.Voltages)
                {
                    if (voltagePhasorGroup.InternalID == m_editedVoltagePhasor.InternalID)
                    {
                        voltagePhasorGroup.Number = m_editedVoltagePhasor.Number;
                        voltagePhasorGroup.Name = m_editedVoltagePhasor.Name;
                        voltagePhasorGroup.Description = m_editedVoltagePhasor.Description;
                        voltagePhasorGroup.IsEnabled = m_editedVoltagePhasor.IsEnabled;
                        voltagePhasorGroup.UseStatusFlagForRemovingMeasurements = m_editedVoltagePhasor.UseStatusFlag;
                        voltagePhasorGroup.PositiveSequence.Measurement.MeasurementVariance = m_editedVoltagePhasor.PositiveSequenceVariance;
                        voltagePhasorGroup.PhaseA.Measurement.MeasurementVariance = m_editedVoltagePhasor.APhaseVariance;
                        voltagePhasorGroup.PhaseB.Measurement.MeasurementVariance = m_editedVoltagePhasor.BPhaseVariance;
                        voltagePhasorGroup.PhaseC.Measurement.MeasurementVariance = m_editedVoltagePhasor.CPhaseVariance;
                        voltagePhasorGroup.PositiveSequence.Measurement.InstrumentTransformerCalibrationSetting = m_editedVoltagePhasor.InstrumentTransformerCalibrationSetting;
                        voltagePhasorGroup.PhaseA.Measurement.InstrumentTransformerCalibrationSetting = m_editedVoltagePhasor.InstrumentTransformerCalibrationSetting;
                        voltagePhasorGroup.PhaseB.Measurement.InstrumentTransformerCalibrationSetting = m_editedVoltagePhasor.InstrumentTransformerCalibrationSetting;
                        voltagePhasorGroup.PhaseC.Measurement.InstrumentTransformerCalibrationSetting = m_editedVoltagePhasor.InstrumentTransformerCalibrationSetting;
                        voltagePhasorGroup.PositiveSequence.Measurement.RCF = m_editedVoltagePhasor.PositiveSequenceRCF;
                        voltagePhasorGroup.PhaseA.Measurement.RCF = m_editedVoltagePhasor.APhaseRCF;
                        voltagePhasorGroup.PhaseB.Measurement.RCF = m_editedVoltagePhasor.BPhaseRCF;
                        voltagePhasorGroup.PhaseC.Measurement.RCF = m_editedVoltagePhasor.CPhaseRCF;
                        voltagePhasorGroup.PositiveSequence.Measurement.PACF = m_editedVoltagePhasor.PositiveSequencePACF;
                        voltagePhasorGroup.PhaseA.Measurement.PACF = m_editedVoltagePhasor.APhasePACF;
                        voltagePhasorGroup.PhaseB.Measurement.PACF = m_editedVoltagePhasor.BPhasePACF;
                        voltagePhasorGroup.PhaseC.Measurement.PACF = m_editedVoltagePhasor.CPhasePACF;
                        voltagePhasorGroup.PositiveSequence.Measurement.MagnitudeKey = m_editedVoltagePhasor.PositiveSequenceMagnitudeMeasurementKey;
                        voltagePhasorGroup.PositiveSequence.Measurement.AngleKey = m_editedVoltagePhasor.PositiveSequenceAngleMeasurementKey;
                        voltagePhasorGroup.PhaseA.Measurement.MagnitudeKey = m_editedVoltagePhasor.APhaseMagnitudeMeasurementKey;
                        voltagePhasorGroup.PhaseA.Measurement.AngleKey = m_editedVoltagePhasor.APhaseAngleMeasurementKey;
                        voltagePhasorGroup.PhaseB.Measurement.MagnitudeKey = m_editedVoltagePhasor.BPhaseMagnitudeMeasurementKey;
                        voltagePhasorGroup.PhaseB.Measurement.AngleKey = m_editedVoltagePhasor.BPhaseAngleMeasurementKey;
                        voltagePhasorGroup.PhaseC.Measurement.MagnitudeKey = m_editedVoltagePhasor.CPhaseMagnitudeMeasurementKey;
                        voltagePhasorGroup.PhaseC.Measurement.AngleKey = m_editedVoltagePhasor.CPhaseAngleMeasurementKey;
                        voltagePhasorGroup.PositiveSequence.Measurement.MagnitudeKey = m_editedVoltagePhasor.PositiveSequenceMagnitudeEstimateKey;
                        voltagePhasorGroup.PositiveSequence.Measurement.AngleKey = m_editedVoltagePhasor.PositiveSequenceAngleEstimateKey;
                        voltagePhasorGroup.PhaseA.Measurement.MagnitudeKey = m_editedVoltagePhasor.APhaseMagnitudeEstimateKey;
                        voltagePhasorGroup.PhaseA.Measurement.AngleKey = m_editedVoltagePhasor.APhaseAngleEstimateKey;
                        voltagePhasorGroup.PhaseB.Measurement.MagnitudeKey = m_editedVoltagePhasor.BPhaseMagnitudeEstimateKey;
                        voltagePhasorGroup.PhaseB.Measurement.AngleKey = m_editedVoltagePhasor.BPhaseAngleEstimateKey;
                        voltagePhasorGroup.PhaseC.Measurement.MagnitudeKey = m_editedVoltagePhasor.CPhaseMagnitudeEstimateKey;
                        voltagePhasorGroup.PhaseC.Measurement.AngleKey = m_editedVoltagePhasor.CPhaseAngleEstimateKey;
                    }
                }

                network.SerializeToXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);

                LoadVoltagePhasorView(LoadNetworkModel());
            }

            m_editedVoltagePhasor = null;
        }

        private void VoltagePhasorDataGrid_CellEditEnding(object sender, DataGridCellEditEndingEventArgs e)
        {
            if (e.EditAction == DataGridEditAction.Commit)
            {
                m_editedVoltagePhasor = e.Row.Item as VoltagePhasorDataModel;
            }
        }

        private void RefreshButton_Click(object sender, RoutedEventArgs e)
        {
            LoadVoltagePhasorView(LoadNetworkModel());
        }

        private Network LoadNetworkModel()
        {
            Network network = Network.DeserializeFromXml(ConfigurationManager.AppSettings["WorkingDirectory"] + "/" + ConfigurationManager.AppSettings["DefaultConfigurationFile"]);
            network.Initialize();
            return network;
        }

        private void LoadVoltagePhasorView(Network network)
        {
            m_voltagePhasorData.Clear();

            foreach (VoltagePhasorGroup voltagePhasorGroup in network.Model.Voltages)
            {
                m_voltagePhasorData.Add(new VoltagePhasorDataModel()
                {
                    InternalID = voltagePhasorGroup.InternalID,
                    Number = voltagePhasorGroup.Number,
                    Acronym = voltagePhasorGroup.Acronym,
                    Name = voltagePhasorGroup.Name,
                    Description = voltagePhasorGroup.Description,
                    IsEnabled = voltagePhasorGroup.IsEnabled,
                    UseStatusFlag = voltagePhasorGroup.UseStatusFlagForRemovingMeasurements,
                    PositiveSequenceVariance = voltagePhasorGroup.PositiveSequence.Measurement.MeasurementVariance,
                    APhaseVariance = voltagePhasorGroup.PhaseA.Measurement.MeasurementVariance,
                    BPhaseVariance = voltagePhasorGroup.PhaseB.Measurement.MeasurementVariance,
                    CPhaseVariance = voltagePhasorGroup.PhaseC.Measurement.MeasurementVariance,
                    InstrumentTransformerCalibrationSetting = voltagePhasorGroup.PositiveSequence.Measurement.InstrumentTransformerCalibrationSetting,
                    PositiveSequenceRCF = voltagePhasorGroup.PositiveSequence.Measurement.RCF,
                    APhaseRCF = voltagePhasorGroup.PhaseA.Measurement.RCF,
                    BPhaseRCF = voltagePhasorGroup.PhaseB.Measurement.RCF,
                    CPhaseRCF = voltagePhasorGroup.PhaseC.Measurement.RCF,
                    PositiveSequencePACF = voltagePhasorGroup.PositiveSequence.Measurement.PACF,
                    APhasePACF = voltagePhasorGroup.PhaseA.Measurement.PACF,
                    BPhasePACF = voltagePhasorGroup.PhaseB.Measurement.PACF,
                    CPhasePACF = voltagePhasorGroup.PhaseC.Measurement.PACF,
                    PositiveSequenceMagnitudeMeasurementKey = voltagePhasorGroup.PositiveSequence.Measurement.MagnitudeKey,
                    PositiveSequenceAngleMeasurementKey = voltagePhasorGroup.PositiveSequence.Measurement.AngleKey,
                    APhaseMagnitudeMeasurementKey = voltagePhasorGroup.PhaseA.Measurement.MagnitudeKey,
                    APhaseAngleMeasurementKey = voltagePhasorGroup.PhaseA.Measurement.AngleKey,
                    BPhaseMagnitudeMeasurementKey = voltagePhasorGroup.PhaseB.Measurement.MagnitudeKey,
                    BPhaseAngleMeasurementKey = voltagePhasorGroup.PhaseB.Measurement.AngleKey,
                    CPhaseMagnitudeMeasurementKey = voltagePhasorGroup.PhaseC.Measurement.MagnitudeKey,
                    CPhaseAngleMeasurementKey = voltagePhasorGroup.PhaseC.Measurement.AngleKey,
                    PositiveSequenceMagnitudeEstimateKey = voltagePhasorGroup.PositiveSequence.Estimate.MagnitudeKey,
                    PositiveSequenceAngleEstimateKey = voltagePhasorGroup.PositiveSequence.Estimate.AngleKey,
                    APhaseMagnitudeEstimateKey = voltagePhasorGroup.PhaseA.Estimate.MagnitudeKey,
                    APhaseAngleEstimateKey = voltagePhasorGroup.PhaseA.Estimate.AngleKey,
                    BPhaseMagnitudeEstimateKey = voltagePhasorGroup.PhaseB.Estimate.MagnitudeKey,
                    BPhaseAngleEstimateKey = voltagePhasorGroup.PhaseB.Estimate.AngleKey,
                    CPhaseMagnitudeEstimateKey = voltagePhasorGroup.PhaseC.Estimate.MagnitudeKey,
                    CPhaseAngleEstimateKey = voltagePhasorGroup.PhaseC.Estimate.AngleKey,
                });
            }

        }
    }
}
